#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
#
# Copyright (C) 2017 Lenovo, Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
# Module to send Conditional CLI commands to Lenovo Switches
# Lenovo Networking
#

DOCUMENTATION = '''
---
module: cnos_conditional_command
author: "Anil Kumar Muraleedharan (@amuraleedhar)"
short_description: Execute a single command based on condition on devices
 running Lenovo CNOS
description:
   - This module allows you to modify the running configuration of a switch. It
    provides a way to execute a single CNOS command on a network device by
    evaluating the current running configuration and executing the command only
    if the specific settings have not been already configured.
    The CNOS command is passed as an argument of the method.
    This module functions the same as the cnos_command module.
    The only exception is that following inventory variable can be specified
    ["condition = <flag string>"]
    When this inventory variable is specified as the variable of a task, the
    command is executed for the network element that matches the flag string.
    Usually, commands are executed across a group of network devices. When
    there is a requirement to skip the execution of the command on one or
    more devices, it is recommended to use this module. This module uses SSH to
    manage network device configuration.
extends_documentation_fragment:
- community.network.cnos

options:
    clicommand:
        description:
            - This specifies the CLI command as an attribute to this method.
             The command is passed using double quotes. The variables can be
             placed directly on to the CLI commands or can be invoked
             from the vars directory.
        required: true
        default: Null
    condition:
        description:
            - If you specify condition=false in the inventory file against any
             device, the command execution is skipped for that device.
        required: true
        default: Null
    flag:
        description:
            - If a task needs to be executed, you have to set the flag the same
             as it is specified in the inventory for that device.
        required: true
        default: Null

'''
EXAMPLES = '''
Tasks : The following are examples of using the module
 cnos_conditional_command. These are written in the main.yml file of the tasks
 directory.
---
- name: Applying CLI template on VLAG Tier1 Leaf Switch1
  community.network.cnos_conditional_command:
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      outputfile: "./results/test_conditional_command_
                  {{ inventory_hostname }}_output.txt"
      condition: "{{ hostvars[inventory_hostname]['condition']}}"
      flag: leaf_switch2
      command: "spanning-tree mode enable"

'''
RETURN = '''
msg:
  description: Success or failure message
  returned: always
  type: str
  sample: "Command Applied"
'''

import sys
import time
import socket
import array
import json
import time
import re
import os
try:
    from ansible_collections.community.network.plugins.module_utils.network.cnos import cnos
    HAS_LIB = True
except Exception:
    HAS_LIB = False
from ansible.module_utils.basic import AnsibleModule
from collections import defaultdict


def main():
    module = AnsibleModule(
        argument_spec=dict(
            clicommand=dict(required=True),
            outputfile=dict(required=True),
            condition=dict(required=True),
            flag=dict(required=True),
            host=dict(required=False),
            deviceType=dict(required=True),
            username=dict(required=False),
            password=dict(required=False, no_log=True),
            enablePassword=dict(required=False,
                                no_log=True), ), supports_check_mode=False)

    condition = module.params['condition']
    flag = module.params['flag']
    cliCommand = module.params['clicommand']
    outputfile = module.params['outputfile']
    output = ''
    if (condition is None or condition != flag):
        module.exit_json(changed=True, msg="Command Skipped for this switch")
        return ''
    # Send the CLi command
    cmd = [{'command': cliCommand, 'prompt': None, 'answer': None}]
    output = output + str(cnos.run_cnos_commands(module, cmd))
    # Write to memory
    save_cmd = [{'command': 'save', 'prompt': None, 'answer': None}]
    cmd.extend(save_cmd)
    output = output + str(cnos.run_cnos_commands(module, cmd))

    # Save it into the file
    path = outputfile.rsplit('/', 1)
    # cnos.debugOutput(path[0])
    if not os.path.exists(path[0]):
        os.makedirs(path[0])
    file = open(outputfile, "a")
    file.write(output)
    file.close()

    # Logic to check when changes occur or not
    errorMsg = cnos.checkOutputForError(output)
    if(errorMsg is None):
        module.exit_json(changed=True,
                         msg="CLI Command executed and results saved in file ")
    else:
        module.fail_json(msg=errorMsg)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This Ansible library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this library.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


DOCUMENTATION = r'''
---
module: elb_classic_lb_info
version_added: 1.0.0
short_description: Gather information about EC2 Elastic Load Balancers in AWS
description:
    - Gather information about EC2 Elastic Load Balancers in AWS
    - This module was called C(elb_classic_lb_facts) before Ansible 2.9. The usage did not change.
author:
  - "Michael Schultz (@mjschultz)"
  - "Fernando Jose Pando (@nand0p)"
options:
  names:
    description:
      - List of ELB names to gather information about. Pass this option to gather information about a set of ELBs, otherwise, all ELBs are returned.
    type: list
    elements: str
extends_documentation_fragment:
- amazon.aws.aws
- amazon.aws.ec2

requirements:
  - botocore
  - boto3
'''

EXAMPLES = r'''
# Note: These examples do not set authentication details, see the AWS Guide for details.
# Output format tries to match amazon.aws.ec2_elb_lb module input parameters

# Gather information about all ELBs
- community.aws.elb_classic_lb_info:
  register: elb_info

- ansible.builtin.debug:
    msg: "{{ item.dns_name }}"
  loop: "{{ elb_info.elbs }}"

# Gather information about a particular ELB
- community.aws.elb_classic_lb_info:
    names: frontend-prod-elb
  register: elb_info

- ansible.builtin.debug:
    msg: "{{ elb_info.elbs.0.dns_name }}"

# Gather information about a set of ELBs
- community.aws.elb_classic_lb_info:
    names:
    - frontend-prod-elb
    - backend-prod-elb
  register: elb_info

- ansible.builtin.debug:
    msg: "{{ item.dns_name }}"
  loop: "{{ elb_info.elbs }}"

'''

RETURN = r'''
elbs:
  description: a list of load balancers
  returned: always
  type: list
  sample:
    elbs:
      - attributes:
          access_log:
            enabled: false
          connection_draining:
            enabled: true
            timeout: 300
          connection_settings:
            idle_timeout: 60
          cross_zone_load_balancing:
            enabled: true
        availability_zones:
          - "us-east-1a"
          - "us-east-1b"
          - "us-east-1c"
          - "us-east-1d"
          - "us-east-1e"
        backend_server_description: []
        canonical_hosted_zone_name: test-lb-XXXXXXXXXXXX.us-east-1.elb.amazonaws.com
        canonical_hosted_zone_name_id: XXXXXXXXXXXXXX
        created_time: '2017-08-23T18:25:03.280000+00:00'
        dns_name: test-lb-XXXXXXXXXXXX.us-east-1.elb.amazonaws.com
        health_check:
          healthy_threshold: 10
          interval: 30
          target: HTTP:80/index.html
          timeout: 5
          unhealthy_threshold: 2
        instances: []
        instances_inservice: []
        instances_inservice_count: 0
        instances_outofservice: []
        instances_outofservice_count: 0
        instances_unknownservice: []
        instances_unknownservice_count: 0
        listener_descriptions:
          - listener:
              instance_port: 80
              instance_protocol: HTTP
              load_balancer_port: 80
              protocol: HTTP
            policy_names: []
        load_balancer_name: test-lb
        policies:
          app_cookie_stickiness_policies: []
          lb_cookie_stickiness_policies: []
          other_policies: []
        scheme: internet-facing
        security_groups:
          - sg-29d13055
        source_security_group:
          group_name: default
          owner_alias: XXXXXXXXXXXX
        subnets:
          - subnet-XXXXXXXX
          - subnet-XXXXXXXX
        tags: {}
        vpc_id: vpc-c248fda4
'''

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import (
    AWSRetry,
    camel_dict_to_snake_dict,
    boto3_tag_list_to_ansible_dict
)

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule


@AWSRetry.backoff(tries=5, delay=5, backoff=2.0)
def list_elbs(connection, names):
    paginator = connection.get_paginator('describe_load_balancers')
    load_balancers = paginator.paginate(LoadBalancerNames=names).build_full_result().get('LoadBalancerDescriptions', [])
    results = []

    for lb in load_balancers:
        description = camel_dict_to_snake_dict(lb)
        name = lb['LoadBalancerName']
        instances = lb.get('Instances', [])
        description['tags'] = get_tags(connection, name)
        description['instances_inservice'], description['instances_inservice_count'] = lb_instance_health(connection, name, instances, 'InService')
        description['instances_outofservice'], description['instances_outofservice_count'] = lb_instance_health(connection, name, instances, 'OutOfService')
        description['instances_unknownservice'], description['instances_unknownservice_count'] = lb_instance_health(connection, name, instances, 'Unknown')
        description['attributes'] = get_lb_attributes(connection, name)
        results.append(description)
    return results


def get_lb_attributes(connection, name):
    attributes = connection.describe_load_balancer_attributes(LoadBalancerName=name).get('LoadBalancerAttributes', {})
    return camel_dict_to_snake_dict(attributes)


def get_tags(connection, load_balancer_name):
    tags = connection.describe_tags(LoadBalancerNames=[load_balancer_name])['TagDescriptions']
    if not tags:
        return {}
    return boto3_tag_list_to_ansible_dict(tags[0]['Tags'])


def lb_instance_health(connection, load_balancer_name, instances, state):
    instance_states = connection.describe_instance_health(LoadBalancerName=load_balancer_name, Instances=instances).get('InstanceStates', [])
    instate = [instance['InstanceId'] for instance in instance_states if instance['State'] == state]
    return instate, len(instate)


def main():
    argument_spec = dict(
        names={'default': [], 'type': 'list', 'elements': 'str'}
    )
    module = AnsibleAWSModule(argument_spec=argument_spec,
                              supports_check_mode=True)
    if module._name == 'elb_classic_lb_facts':
        module.deprecate("The 'elb_classic_lb_facts' module has been renamed to 'elb_classic_lb_info'", date='2021-12-01', collection_name='community.aws')

    connection = module.client('elb')

    try:
        elbs = list_elbs(connection, module.params.get('names'))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to get load balancer information.")

    module.exit_json(elbs=elbs)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
#
# Copyright (C) 2017 Lenovo, Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
# Module to Rollback Config back to Lenovo Switches
#
# Lenovo Networking
#

DOCUMENTATION = '''
---
module: cnos_rollback
author: "Anil Kumar Muraleedharan (@amuraleedhar)"
short_description: Roll back the running or startup configuration from a remote
 server on devices running Lenovo CNOS
description:
    - This module allows you to work with switch configurations. It provides a
     way to roll back configurations of a switch from a remote server. This is
     achieved by using startup or running configurations of the target device
     that were previously backed up to a remote server using FTP, SFTP, TFTP,
     or SCP. The first step is to create a directory from where the remote
     server can be reached. The next step is to provide the full file path of
     he backup configuration's location. Authentication details required by the
     remote server must be provided as well.
     By default, this method overwrites the switch's configuration file with
     the newly downloaded file. This module uses SSH to manage network device
     configuration. The results of the operation will be placed in a directory
     named 'results' that must be created by the user in their local directory
     to where the playbook is run.
extends_documentation_fragment:
- community.network.cnos

options:
   configType:
        description:
            - This refers to the type of configuration which will be used for
             the rolling back process. The choices are the running or startup
             configurations. There is no default value, so it will result
             in an error if the input is incorrect.
        required: Yes
        default: Null
        choices: [running-config, startup-config]
   protocol:
        description:
            - This refers to the protocol used by the network device to
              interact with the remote server from where to download the backup
              configuration. The choices are FTP, SFTP, TFTP, or SCP. Any other
              protocols will result in error. If this parameter is not
              specified, there is no default value to be used.
        required: Yes
        default: Null
        choices: [SFTP, SCP, FTP, TFTP]
   rcserverip:
        description:
            - This specifies the IP Address of the remote server from where the
             backup configuration will be downloaded.
        required: Yes
        default: Null
   rcpath:
        description:
            - This specifies the full file path of the configuration file
             located on the remote server. In case the relative path is used as
             the variable value, the root folder for the user of the server
             needs to be specified.
        required: Yes
        default: Null
   serverusername:
        description:
            - Specify username for the server relating to the protocol used.
        required: Yes
        default: Null
   serverpassword:
        description:
            - Specify password for the server relating to the protocol used.
        required: Yes
        default: Null
'''
EXAMPLES = '''
Tasks : The following are examples of using the module cnos_rollback.
 These are written in the main.yml file of the tasks directory.
---

- name: Test Rollback of config - Running config
  cnos_rolback:
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      outputfile: "./results/test_rollback_{{ inventory_hostname }}_output.txt"
      configType: running-config
      protocol: "sftp"
      serverip: "10.241.106.118"
      rcpath: "/root/cnos/G8272-running-config.txt"
      serverusername: "root"
      serverpassword: "root123"

- name: Test Rollback of config - Startup config
  cnos_rolback:
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      outputfile: "./results/test_rollback_{{ inventory_hostname }}_output.txt"
      configType: startup-config
      protocol: "sftp"
      serverip: "10.241.106.118"
      rcpath: "/root/cnos/G8272-startup-config.txt"
      serverusername: "root"
      serverpassword: "root123"

- name: Test Rollback of config - Running config - TFTP
  cnos_rolback:
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      outputfile: "./results/test_rollback_{{ inventory_hostname }}_output.txt"
      configType: running-config
      protocol: "tftp"
      serverip: "10.241.106.118"
      rcpath: "/anil/G8272-running-config.txt"
      serverusername: "root"
      serverpassword: "root123"

- name: Test Rollback of config - Startup config - TFTP
  cnos_rolback:
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      outputfile: "./results/test_rollback_{{ inventory_hostname }}_output.txt"
      configType: startup-config
      protocol: "tftp"
      serverip: "10.241.106.118"
      rcpath: "/anil/G8272-startup-config.txt"
      serverusername: "root"
      serverpassword: "root123"

'''
RETURN = '''
msg:
  description: Success or failure message
  returned: always
  type: str
  sample: "Config file transferred to Device"
'''

import sys
import time
import socket
import array
import json
import time
import re
import os
try:
    from ansible_collections.community.network.plugins.module_utils.network.cnos import cnos
    HAS_LIB = True
except Exception:
    HAS_LIB = False
from ansible.module_utils.basic import AnsibleModule
from collections import defaultdict


# Utility Method to rollback the running config or start up config
# This method supports only SCP or SFTP or FTP or TFTP
def doConfigRollBack(module, prompt, answer):
    host = module.params['host']
    server = module.params['serverip']
    username = module.params['serverusername']
    password = module.params['serverpassword']
    protocol = module.params['protocol'].lower()
    rcPath = module.params['rcpath']
    configType = module.params['configType']
    confPath = rcPath
    retVal = ''

    command = "copy " + protocol + " " + protocol + "://"
    command = command + username + "@" + server + "/" + confPath
    command = command + " " + configType + " vrf management\n"
    cnos.debugOutput(command + "\n")
    # cnos.checkForFirstTimeAccess(module, command, 'yes/no', 'yes')
    cmd = []
    if(protocol == "scp"):
        scp_cmd1 = [{'command': command, 'prompt': 'timeout:', 'answer': '0'}]
        scp_cmd2 = [{'command': '\n', 'prompt': 'Password:',
                     'answer': password}]
        cmd.extend(scp_cmd1)
        cmd.extend(scp_cmd2)
        if(configType == 'startup-config'):
            scp_cmd3 = [{'command': 'y', 'prompt': None, 'answer': None}]
            cmd.extend(scp_cmd3)
        retVal = retVal + str(cnos.run_cnos_commands(module, cmd))
    elif(protocol == "sftp"):
        sftp_cmd = [{'command': command, 'prompt': 'Password:',
                     'answer': password}]
        cmd.extend(sftp_cmd)
        # cnos.debugOutput(configType + "\n")
        if(configType == 'startup-config'):
            sftp_cmd2 = [{'command': 'y', 'prompt': None, 'answer': None}]
            cmd.extend(sftp_cmd2)
        retVal = retVal + str(cnos.run_cnos_commands(module, cmd))
    elif(protocol == "ftp"):
        ftp_cmd = [{'command': command, 'prompt': 'Password:',
                    'answer': password}]
        cmd.extend(ftp_cmd)
        if(configType == 'startup-config'):
            ftp_cmd2 = [{'command': 'y', 'prompt': None, 'answer': None}]
            cmd.extend(ftp_cmd2)
        retVal = retVal + str(cnos.run_cnos_commands(module, cmd))
    elif(protocol == "tftp"):
        command = "copy " + protocol + " " + protocol
        command = command + "://" + server + "/" + confPath
        command = command + " " + configType + " vrf management\n"
        cnos.debugOutput(command)
        tftp_cmd = [{'command': command, 'prompt': None, 'answer': None}]
        cmd.extend(tftp_cmd)
        if(configType == 'startup-config'):
            tftp_cmd2 = [{'command': 'y', 'prompt': None, 'answer': None}]
            cmd.extend(tftp_cmd2)
        retVal = retVal + str(cnos.run_cnos_commands(module, cmd))
    else:
        return "Error-110"

    return retVal
# EOM


def main():
    module = AnsibleModule(
        argument_spec=dict(
            outputfile=dict(required=True),
            host=dict(required=False),
            username=dict(required=False),
            password=dict(required=False, no_log=True),
            enablePassword=dict(required=False, no_log=True),
            deviceType=dict(required=True),
            configType=dict(required=True),
            protocol=dict(required=True),
            serverip=dict(required=True),
            rcpath=dict(required=True),
            serverusername=dict(required=False),
            serverpassword=dict(required=False, no_log=True),),
        supports_check_mode=False)

    outputfile = module.params['outputfile']
    protocol = module.params['protocol'].lower()
    output = ''
    if protocol in ('tftp', 'ftp', 'sftp', 'scp'):
        transfer_status = doConfigRollBack(module, None, None)
    else:
        transfer_status = 'Invalid Protocol option'
    output = output + "\n Config Transfer status \n" + transfer_status

    # Save it into the file
    if '/' in outputfile:
        path = outputfile.rsplit('/', 1)
        # cnos.debugOutput(path[0])
        if not os.path.exists(path[0]):
            os.makedirs(path[0])
    file = open(outputfile, "a")
    file.write(output)
    file.close()

    # need to add logic to check when changes occur or not
    errorMsg = cnos.checkOutputForError(output)
    if(errorMsg is None):
        module.exit_json(changed=True, msg="Config file transferred to Device")
    else:
        module.fail_json(msg=errorMsg)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

#
# Dell EMC OpenManage Ansible Modules
# Version 2.0
# Copyright (C) 2019 Dell Inc.

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# All rights reserved. Dell, EMC, and other trademarks are trademarks of Dell Inc. or its subsidiaries.
# Other trademarks may be trademarks of their respective owners.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest
from ansible_collections.community.general.tests.unit.plugins.modules.utils import set_module_args, exit_json, \
    fail_json, AnsibleFailJson, AnsibleExitJson
from ansible.module_utils import basic
from ansible_collections.community.general.plugins.modules.remote_management.dellemc import ome_device_info
from ansible.module_utils.six.moves.urllib.error import HTTPError

default_args = {'hostname': '192.168.0.1', 'username': 'username', 'password': 'password'}
resource_basic_inventory = {"basic_inventory": "DeviceService/Devices"}
resource_detailed_inventory = {"detailed_inventory:": {"device_id": {1234: None},
                                                       "device_service_tag": {1345: "MXL1234"}}}


class TestOmeDeviceInfo(object):
    module = ome_device_info

    @pytest.fixture(autouse=True)
    def module_mock(self, mocker):
        return mocker.patch.multiple(basic.AnsibleModule, exit_json=exit_json, fail_json=fail_json)

    @pytest.fixture
    def connection_mock(self, mocker):
        connection_class_mock = mocker.patch('ansible_collections.community.general.plugins.modules.remote_management.dellemc.ome_device_info.RestOME')
        return connection_class_mock.return_value

    @pytest.fixture
    def response_mock(self, mocker):
        response_class_mock = mocker.patch('ansible_collections.community.general.plugins.module_utils.remote_management.dellemc.ome.OpenURLResponse')
        return response_class_mock

    @pytest.fixture
    def validate_inputs_mock(self, mocker):
        response_class_mock = mocker.patch('ansible_collections.community.general.plugins.modules.remote_management.dellemc.ome_device_info._validate_inputs')
        response_class_mock.return_value = None

    @pytest.fixture
    def get_device_identifier_map_mock(self, mocker):
        response_class_mock = mocker.patch(
            'ansible_collections.community.general.plugins.modules.remote_management.dellemc.ome_device_info._get_device_identifier_map'
        )
        response_class_mock.return_value = resource_detailed_inventory
        return response_class_mock.return_value

    @pytest.fixture
    def get_resource_parameters_mock(self, mocker):
        response_class_mock = mocker.patch(
            'ansible_collections.community.general.plugins.modules.remote_management.dellemc.ome_device_info._get_resource_parameters'
        )
        return response_class_mock

    def test_main_basic_inventory_success_case(self, module_mock, validate_inputs_mock, connection_mock, get_resource_parameters_mock, response_mock):
        get_resource_parameters_mock.return_value = resource_basic_inventory
        connection_mock.__enter__.return_value = connection_mock
        connection_mock.invoke_request.return_value = response_mock
        response_mock.json_data = {"value": [{"device_id1": "details", "device_id2": "details"}]}
        response_mock.status_code = 200
        result = self._run_module(default_args)
        assert result['changed'] is False
        assert 'device_info' in result

    def test_main_basic_inventory_failure_case(self, module_mock, validate_inputs_mock, connection_mock, get_resource_parameters_mock, response_mock):
        get_resource_parameters_mock.return_value = resource_basic_inventory
        connection_mock.__enter__.return_value = connection_mock
        connection_mock.invoke_request.return_value = response_mock
        response_mock.status_code = 500
        result = self._run_module_with_fail_json(default_args)
        assert result['msg'] == 'Failed to fetch the device information'

    def test_main_detailed_inventory_success_case(self, module_mock, validate_inputs_mock, connection_mock, get_resource_parameters_mock, response_mock):
        default_args.update({"fact_subset": "detailed_inventory", "system_query_options": {"device_id": [1234], "device_service_tag": ["MXL1234"]}})
        detailed_inventory = {"detailed_inventory:": {"device_id": {1234: "DeviceService/Devices(1234)/InventoryDetails"},
                                                      "device_service_tag": {"MXL1234": "DeviceService/Devices(4321)/InventoryDetails"}}}
        get_resource_parameters_mock.return_value = detailed_inventory
        connection_mock.__enter__.return_value = connection_mock
        connection_mock.invoke_request.return_value = response_mock
        response_mock.json_data = {"value": [{"device_id": {"1234": "details"}}, {"device_service_tag": {"MXL1234": "details"}}]}
        response_mock.status_code = 200
        result = self._run_module(default_args)
        assert result['changed'] is False
        assert 'device_info' in result

    def test_main_HTTPError_error_case(self, module_mock, validate_inputs_mock, connection_mock, get_resource_parameters_mock, response_mock):
        get_resource_parameters_mock.return_value = resource_basic_inventory
        connection_mock.__enter__.return_value = connection_mock
        connection_mock.invoke_request.side_effect = HTTPError('http://testhost.com', 400, '', {}, None)
        response_mock.json_data = {"value": [{"device_id1": "details", "device_id2": "details"}]}
        response_mock.status_code = 400
        result = self._run_module_with_fail_json(default_args)
        assert 'device_info' not in result
        assert result['failed'] is True

    @pytest.mark.parametrize("fact_subset, mutually_exclusive_call", [("basic_inventory", False), ("detailed_inventory", True)])
    def test_validate_inputs(self, fact_subset, mutually_exclusive_call, mocker):
        module_params = {"fact_subset": fact_subset}
        check_mutually_inclusive_arguments_mock = mocker.patch(
            'ansible_collections.community.general.plugins.modules.remote_management.dellemc.ome_device_info._check_mutually_inclusive_arguments')
        check_mutually_inclusive_arguments_mock.return_value = None
        self.module._validate_inputs(module_params)
        if mutually_exclusive_call:
            check_mutually_inclusive_arguments_mock.assert_called()
        else:
            check_mutually_inclusive_arguments_mock.assert_not_called()
        check_mutually_inclusive_arguments_mock.reset_mock()

    system_query_options_params = [{"system_query_options": None}, {"system_query_options": {"device_id": None}},
                                   {"system_query_options": {"device_service_tag": None}}]

    @pytest.mark.parametrize("system_query_options_params", system_query_options_params)
    def test_check_mutually_inclusive_arguments(self, system_query_options_params):
        module_params = {"fact_subset": "subsystem_health"}
        required_args = ["device_id", "device_service_tag"]
        module_params.update(system_query_options_params)
        with pytest.raises(ValueError) as ex:
            self.module._check_mutually_inclusive_arguments(module_params["fact_subset"], module_params, ["device_id", "device_service_tag"])
        assert "One of the following {0} is required for {1}".format(required_args, module_params["fact_subset"]) == str(ex.value)

    params = [{"fact_subset": "basic_inventory", "system_query_options": {"device_id": [1234]}},
              {"fact_subset": "subsystem_health", "system_query_options": {"device_service_tag": ["MXL1234"]}},
              {"fact_subset": "detailed_inventory", "system_query_options": {"device_id": [1234], "inventory_type": "serverDeviceCards"}}]

    @pytest.mark.parametrize("module_params", params)
    def test_get_resource_parameters(self, module_params, connection_mock):
        self.module._get_resource_parameters(module_params, connection_mock)

    @pytest.mark.parametrize("module_params,data", [({"system_query_options": None}, None), ({"system_query_options": {"fileter": None}}, None),
                                                    ({"system_query_options": {"filter": "abc"}}, "$filter")])
    def test_get_query_parameters(self, module_params, data):
        res = self.module._get_query_parameters(module_params)
        if data is not None:
            assert data in res
        else:
            assert res is None

    @pytest.mark.parametrize("module_params", params)
    def test_get_device_identifier_map(self, module_params, connection_mock, mocker):
        get_device_id_from_service_tags_mock = mocker.patch(
            'ansible_collections.community.general.plugins.modules.remote_management.dellemc.ome_device_info._get_device_id_from_service_tags'
        )
        get_device_id_from_service_tags_mock.return_value = None
        res = self.module._get_device_identifier_map(module_params, connection_mock)
        assert isinstance(res, dict)

    def test_check_duplicate_device_id(self):
        self.module._check_duplicate_device_id([1234], {1234: "MX1234"})
        assert self.module.device_fact_error_report["MX1234"] == "Duplicate report of device_id: 1234"

    @pytest.mark.parametrize("val,expected_res", [(123, True), ("abc", False)])
    def test_is_int(self, val, expected_res):
        actual_res = self.module.is_int(val)
        assert actual_res == expected_res

    def test_get_device_id_from_service_tags(self, connection_mock, response_mock):
        connection_mock.__enter__.return_value = connection_mock
        connection_mock.invoke_request.return_value = response_mock
        response_mock.json_data = {"value": [{"DeviceServiceTag": "MX1234", "Id": 1234}]}
        response_mock.status_code = 200
        response_mock.success = True
        self.module._get_device_id_from_service_tags(["MX1234", "INVALID"], connection_mock)

    def test_get_device_id_from_service_tags_error_case(self, connection_mock, response_mock):
        connection_mock.__enter__.return_value = connection_mock
        connection_mock.invoke_request.side_effect = HTTPError('http://testhost.com',
                                                               400, '', {}, None)
        response_mock.json_data = {"value": [{"DeviceServiceTag": "MX1234", "Id": 1234}]}
        response_mock.status_code = 200
        response_mock.success = True
        with pytest.raises(HTTPError) as ex:
            self.module._get_device_id_from_service_tags(["INVALID"], connection_mock)

    def _run_module(self, module_args):
        set_module_args(module_args)
        with pytest.raises(AnsibleExitJson) as ex:
            self.module.main()
        return ex.value.args[0]

    def _run_module_with_fail_json(self, module_args):
        set_module_args(module_args)
        with pytest.raises(AnsibleFailJson) as exc:
            self.module.main()
        result = exc.value.args[0]
        return result



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#


from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "network",
}


DOCUMENTATION = """module: checkpoint_task_facts
short_description: Get task objects facts on Check Point over Web Services API
description:
- Get task objects facts on Check Point devices. All operations are performed over
  Web Services API.
version_added: "2.7"
author: Ansible by Red Hat (@rcarrillocruz)
options:
  task_id:
    description:
    - ID of the task object.
    type: str
    required: true
"""

EXAMPLES = """
- name: Get task facts
  checkpoint_task_facts:
    task_id: 2eec70e5-78a8-4bdb-9a76-cfb5601d0bcb
"""

RETURN = """
ansible_facts:
  description: The checkpoint task facts.
  returned: always.
  type: list
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection


def get_task(module, connection):
    task_id = module.params["task_id"]

    if task_id:
        payload = {"task-id": task_id, "details-level": "full"}

        code, response = connection.send_request("/web_api/show-task", payload)
    else:
        code, response = connection.send_request("/web_api/show-tasks", None)

    return code, response


def main():
    argument_spec = dict(task_id=dict(type="str", required=True))

    module = AnsibleModule(argument_spec=argument_spec)
    connection = Connection(module._socket_path)
    code, response = get_task(module, connection)
    if code == 200:
        module.exit_json(ansible_facts=dict(checkpoint_tasks=response))
    else:
        module.fail_json(
            msg="Checkpoint device returned error {0} with message {1}".format(
                code, response
            )
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2017, Kairo Araujo <kairo@kairo.eti.br>
# GNU General Public License v3.0+ (see COPYING or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
author: Kairo Araujo (@kairoaraujo)
module: mksysb
short_description: Generates AIX mksysb rootvg backups.
description:
  - This module manages a basic AIX mksysb (image) of rootvg.
options:
  backup_crypt_files:
    description:
      - Backup encrypted files.
    type: bool
    default: "yes"
  backup_dmapi_fs:
    description:
      - Back up DMAPI filesystem files.
    type: bool
    default: "yes"
  create_map_files:
    description:
      - Creates a new MAP files.
    type: bool
    default: "no"
  exclude_files:
    description:
      - Excludes files using C(/etc/rootvg.exclude).
    type: bool
    default: "no"
  exclude_wpar_files:
    description:
      - Excludes WPAR files.
    type: bool
    default: "no"
  extended_attrs:
    description:
      - Backup extended attributes.
    type: bool
    default: "yes"
  name:
    description:
      - Backup name
    required: true
  new_image_data:
    description:
      - Creates a new file data.
    type: bool
    default: "yes"
  software_packing:
    description:
      - Exclude files from packing option listed in
        C(/etc/exclude_packing.rootvg).
    type: bool
    default: "no"
  storage_path:
    description:
      - Storage path where the mksysb will stored.
    required: true
  use_snapshot:
    description:
      - Creates backup using snapshots.
    type: bool
    default: "no"
'''

EXAMPLES = '''
- name: Running a backup image mksysb
  community.general.mksysb:
    name: myserver
    storage_path: /repository/images
    exclude_files: yes
    exclude_wpar_files: yes
'''

RETURN = '''
changed:
  description: Return changed for mksysb actions as true or false.
  returned: always
  type: bool
msg:
  description: Return message regarding the action.
  returned: always
  type: str
'''


from ansible.module_utils.basic import AnsibleModule
import os


def main():
    module = AnsibleModule(
        argument_spec=dict(
            backup_crypt_files=dict(type='bool', default=True),
            backup_dmapi_fs=dict(type='bool', default=True),
            create_map_files=dict(type='bool', default=False),
            exclude_files=dict(type='bool', default=False),
            exclude_wpar_files=dict(type='bool', default=False),
            extended_attrs=dict(type='bool', default=True),
            name=dict(required=True),
            new_image_data=dict(type='bool', default=True),
            software_packing=dict(type='bool', default=False),
            storage_path=dict(required=True),
            use_snapshot=dict(type='bool', default=False)
        ),
        supports_check_mode=True,
    )

    # Command options.
    map_file_opt = {
        True: '-m',
        False: ''
    }

    use_snapshot_opt = {
        True: '-T',
        False: ''
    }

    exclude_files_opt = {
        True: '-e',
        False: ''
    }

    exclude_wpar_opt = {
        True: '-G',
        False: ''
    }

    new_image_data_opt = {
        True: '-i',
        False: ''
    }

    soft_packing_opt = {
        True: '',
        False: '-p'
    }

    extend_attr_opt = {
        True: '',
        False: '-a'
    }

    crypt_files_opt = {
        True: '',
        False: '-Z'
    }

    dmapi_fs_opt = {
        True: '-a',
        False: ''
    }

    backup_crypt_files = crypt_files_opt[module.params['backup_crypt_files']]
    backup_dmapi_fs = dmapi_fs_opt[module.params['backup_dmapi_fs']]
    create_map_files = map_file_opt[module.params['create_map_files']]
    exclude_files = exclude_files_opt[module.params['exclude_files']]
    exclude_wpar_files = exclude_wpar_opt[module.params['exclude_wpar_files']]
    extended_attrs = extend_attr_opt[module.params['extended_attrs']]
    name = module.params['name']
    new_image_data = new_image_data_opt[module.params['new_image_data']]
    software_packing = soft_packing_opt[module.params['software_packing']]
    storage_path = module.params['storage_path']
    use_snapshot = use_snapshot_opt[module.params['use_snapshot']]

    # Validate if storage_path is a valid directory.
    if os.path.isdir(storage_path):
        if not module.check_mode:
            # Generates the mksysb image backup.
            mksysb_cmd = module.get_bin_path('mksysb', True)
            rc, mksysb_output, err = module.run_command(
                "%s -X %s %s %s %s %s %s %s %s %s %s/%s" % (
                    mksysb_cmd, create_map_files, use_snapshot, exclude_files,
                    exclude_wpar_files, software_packing, extended_attrs,
                    backup_crypt_files, backup_dmapi_fs, new_image_data,
                    storage_path, name))
            if rc == 0:
                module.exit_json(changed=True, msg=mksysb_output)
            else:
                module.fail_json(msg="mksysb failed.", rc=rc, err=err)

        module.exit_json(changed=True)

    else:
        module.fail_json(msg="Storage path %s is not valid." % storage_path)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: eos_static_route
author: Trishna Guha (@trishnaguha)
short_description: (deprecated, removed after 2022-06-01) Manage static IP
  routes on Arista EOS network devices
description:
- This module provides declarative management of static IP routes on Arista EOS network
  devices.
version_added: 1.0.0
deprecated:
  alternative: eos_static_routes
  why: Updated modules with more functionality
  removed_at_date: '2022-06-01'
notes:
- Tested against EOS 4.15
options:
  address:
    description:
    - Network address with prefix of the static route.
    aliases:
    - prefix
    type: str
  next_hop:
    description:
    - Next hop IP of the static route.
    type: str
  vrf:
    description:
    - VRF for static route.
    default: default
    type: str
  admin_distance:
    description:
    - Admin distance of the static route.
    default: 1
    type: int
  aggregate:
    description: List of static route definitions
    type: list
    elements: dict
    suboptions:
      address:
        description:
        - Network address with prefix of the static route.
        aliases:
        - prefix
        type: str
        required: True
      next_hop:
        description:
        - Next hop IP of the static route.
        type: str
      vrf:
        description:
        - VRF for static route.
        default: default
        type: str
      admin_distance:
        description:
        - Admin distance of the static route.
        type: int
      state:
        description:
        - State of the static route configuration.
        choices:
        - present
        - absent
        type: str
  state:
    description:
    - State of the static route configuration.
    default: present
    choices:
    - present
    - absent
    type: str
extends_documentation_fragment:
- arista.eos.eos

"""

EXAMPLES = """
- name: configure static route
  arista.eos.eos_static_route:
    address: 10.0.2.0/24
    next_hop: 10.8.38.1
    admin_distance: 2
- name: delete static route
  arista.eos.eos_static_route:
    address: 10.0.2.0/24
    next_hop: 10.8.38.1
    state: absent
- name: configure static routes using aggregate
  arista.eos.eos_static_route:
    aggregate:
    - {address: 10.0.1.0/24, next_hop: 10.8.38.1}
    - {address: 10.0.3.0/24, next_hop: 10.8.38.1}
- name: Delete static route using aggregate
  arista.eos.eos_static_route:
    aggregate:
    - {address: 10.0.1.0/24, next_hop: 10.8.38.1}
    - {address: 10.0.3.0/24, next_hop: 10.8.38.1}
    state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - ip route 10.0.2.0/24 10.8.38.1 3
    - no ip route 10.0.2.0/24 10.8.38.1
"""

import re

from copy import deepcopy

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    is_masklen,
    validate_ip_address,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
    validate_prefix,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import (
    get_config,
    load_config,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import (
    eos_argument_spec,
)


def is_address(value):
    if value:
        address = value.split("/")
        if is_masklen(address[1]) and validate_ip_address(address[0]):
            return True
    return False


def is_hop(value):
    if value:
        if validate_ip_address(value):
            return True
    return False


def map_obj_to_commands(updates, module):
    commands = list()
    want, have = updates

    for w in want:
        address = w["address"]
        next_hop = w["next_hop"]
        admin_distance = w["admin_distance"]
        vrf = w["vrf"]
        state = w["state"]
        del w["state"]

        if state == "absent" and w in have:
            if vrf == "default":
                commands.append("no ip route %s %s" % (address, next_hop))
            else:
                commands.append(
                    "no ip route vrf %s %s %s" % (vrf, address, next_hop)
                )
        elif state == "present" and w not in have:
            if vrf == "default":
                commands.append(
                    "ip route %s %s %d" % (address, next_hop, admin_distance)
                )
            else:
                commands.append(
                    "ip route vrf %s %s %s %d"
                    % (vrf, address, next_hop, admin_distance)
                )

    return commands


def map_params_to_obj(module, required_together=None):
    obj = []

    aggregate = module.params.get("aggregate")
    if aggregate:
        for item in aggregate:
            for key in item:
                if item.get(key) is None:
                    item[key] = module.params[key]

            module._check_required_together(required_together, item)
            d = item.copy()

            obj.append(d)
    else:
        obj.append(
            {
                "address": module.params["address"].strip(),
                "next_hop": module.params["next_hop"].strip(),
                "admin_distance": module.params["admin_distance"],
                "state": module.params["state"],
                "vrf": module.params["vrf"],
            }
        )

    return obj


def map_config_to_obj(module):
    objs = []

    try:
        out = get_config(module, flags=["| include ip.route"])
    except IndexError:
        out = ""
    if out:
        lines = out.splitlines()
        for line in lines:
            obj = {}
            add_match = re.search(r"ip route ([\d\./]+)", line, re.M)
            if add_match:
                obj["vrf"] = "default"
                address = add_match.group(1)
                if is_address(address):
                    obj["address"] = address
                hop_match = re.search(
                    r"ip route {0} ([\d\./]+)".format(address), line, re.M
                )
                if hop_match:
                    hop = hop_match.group(1)
                    if is_hop(hop):
                        obj["next_hop"] = hop
                    dist_match = re.search(
                        r"ip route {0} {1} (\d+)".format(address, hop),
                        line,
                        re.M,
                    )
                    if dist_match:
                        distance = dist_match.group(1)
                        obj["admin_distance"] = int(distance)
                    else:
                        obj["admin_distance"] = 1

            vrf_match = re.search(
                r"ip route vrf ([\w]+) ([\d\./]+)", line, re.M
            )
            if vrf_match:
                vrf = vrf_match.group(1)
                obj["vrf"] = vrf
                address = vrf_match.group(2)
                if is_address(address):
                    obj["address"] = address
                hop_vrf_match = re.search(
                    r"ip route vrf {0} {1} ([\d\./]+)".format(vrf, address),
                    line,
                    re.M,
                )
                if hop_vrf_match:
                    hop = hop_vrf_match.group(1)
                    if is_hop(hop):
                        obj["next_hop"] = hop
                    dist_vrf_match = re.search(
                        r"ip route vrf {0} {1} {2} (\d+)".format(
                            vrf, address, hop
                        ),
                        line,
                        re.M,
                    )
                    if dist_vrf_match:
                        distance = dist_vrf_match.group(1)
                        obj["admin_distance"] = int(distance)
                    else:
                        obj["admin_distance"] = 1

            objs.append(obj)

    return objs


def main():
    """ main entry point for module execution
    """
    element_spec = dict(
        address=dict(type="str", aliases=["prefix"]),
        next_hop=dict(type="str"),
        vrf=dict(type="str", default="default"),
        admin_distance=dict(default=1, type="int"),
        state=dict(default="present", choices=["present", "absent"]),
    )

    aggregate_spec = deepcopy(element_spec)
    aggregate_spec["address"] = dict(required=True, aliases=["prefix"])

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)

    aggregate_spec["vrf"].update(default="default")
    argument_spec = dict(
        aggregate=dict(type="list", elements="dict", options=aggregate_spec)
    )

    argument_spec.update(element_spec)
    argument_spec.update(eos_argument_spec)

    required_one_of = [["aggregate", "address"]]
    required_together = [["address", "next_hop"]]
    mutually_exclusive = [["aggregate", "address"]]

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=required_one_of,
        required_together=required_together,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    address = module.params["address"]
    if address is not None:
        prefix = address.split("/")[-1]

    if address and prefix:
        if "/" not in address or not validate_ip_address(
            address.split("/")[0]
        ):
            module.fail_json(
                msg="{0} is not a valid IP address".format(address)
            )

        if not validate_prefix(prefix):
            module.fail_json(
                msg="Length of prefix should be between 0 and 32 bits"
            )

    warnings = list()
    result = {"changed": False}
    if warnings:
        result["warnings"] = warnings

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)
    commands = map_obj_to_commands((want, have), module)
    result["commands"] = commands

    if commands:
        commit = not module.check_mode
        response = load_config(module, commands, commit=commit)
        if response.get("diff") and module._diff:
            result["diff"] = {"prepared": response.get("diff")}
        result["session_name"] = response.get("session")
        result["changed"] = True

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2017 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_containerregistrywebhook
version_added: "2.5"
short_description: Manage Webhook instance.
description:
    - Create, update and delete instance of Webhook.

options:
    resource_group:
        description:
            - The name of the resource group to which the container registry belongs.
        required: True
    registry_name:
        description:
            - The name of the container registry.
        required: True
    webhook_name:
        description:
            - The name of the webhook.
        required: True
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
    service_uri:
        description:
            - The service URI for the webhook to post notifications.
    custom_headers:
        description:
            - Custom headers that will be added to the webhook notifications.
    status:
        description:
            - The status of the webhook at the time the operation was called.
        choices:
            - 'enabled'
            - 'disabled'
    scope:
        description:
            - "The scope of repositories where the event can be triggered. For example, 'foo:*' means events for all tags under repository 'foo'. 'foo:bar' m
              eans events for 'foo:bar' only. 'foo' is equivalent to 'foo:latest'. Empty means all events."
    actions:
        description:
            - The list of actions that trigger the webhook to post notifications.
        type: list

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - "Zim Kalinowski (@zikalino)"

'''

EXAMPLES = '''
  - name: Create (or update) Webhook
    azure_rm_containerregistrywebhook:
      resource_group: myResourceGroup
      registry_name: myRegistry
      webhook_name: myWebhook
      location: eastus
'''

RETURN = '''
id:
    description:
        - The resource ID.
    returned: always
    type: str
    sample: "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/myResourceGroup/providers/Microsoft.ContainerRegistry/registries/myRegistry/w
            ebhooks/myWebhook"
status:
    description:
        - "The status of the webhook at the time the operation was called. Possible values include: 'enabled', 'disabled'"
    returned: always
    type: str
    sample: enabled
'''

import time
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from msrestazure.azure_exceptions import CloudError
    from msrestazure.azure_operation import AzureOperationPoller
    from azure.mgmt.containerregistry import ContainerRegistryManagementClient
    from msrest.serialization import Model
    from msrest.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


def create_webhook_dict(webhook):
    if webhook is None:
        return None
    results = dict(
        id=webhook.id,
        name=webhook.name,
        location=webhook.location,
        provisioning_state=webhook.provisioning_state,
        tags=webhook.tags,
        status=webhook.status
    )
    return results


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMWebhooks(AzureRMModuleBase):
    """Configuration class for an Azure RM Webhook resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            registry_name=dict(
                type='str',
                required=True
            ),
            webhook_name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            service_uri=dict(
                type='str'
            ),
            custom_headers=dict(
                type='dict'
            ),
            status=dict(
                type='str',
                choices=['enabled',
                         'disabled']
            ),
            scope=dict(
                type='str'
            ),
            actions=dict(
                type='list'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.registry_name = None
        self.webhook_name = None
        self.parameters = dict()

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMWebhooks, self).__init__(derived_arg_spec=self.module_arg_spec,
                                              supports_check_mode=True,
                                              supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == "location":
                    self.parameters["location"] = kwargs[key]
                elif key == "service_uri":
                    self.parameters["service_uri"] = kwargs[key]
                elif key == "custom_headers":
                    self.parameters["custom_headers"] = kwargs[key]
                elif key == "status":
                    self.parameters["status"] = kwargs[key]
                elif key == "scope":
                    self.parameters["scope"] = kwargs[key]
                elif key == "actions":
                    self.parameters["actions"] = kwargs[key]

        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(ContainerRegistryManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager,
                                                    api_version='2017-10-01')

        resource_group = self.get_resource_group(self.resource_group)

        if 'location' not in self.parameters:
            self.parameters['location'] = resource_group.location

        old_response = self.get_webhook()

        if not old_response:
            self.log("Webhook instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("Webhook instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                self.log("Need to check if Webhook instance has to be deleted or may be updated")
                self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the Webhook instance")

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            response = self.create_update_webhook()

            if not old_response:
                self.results['changed'] = True
            else:
                self.results['changed'] = old_response.__ne__(response)
            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("Webhook instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_webhook()
            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_webhook():
                time.sleep(20)
        else:
            self.log("Webhook instance unchanged")
            self.results['changed'] = False
            response = old_response

        if response:
            self.results["id"] = response["id"]
            self.results["status"] = response["status"]

        return self.results

    def create_update_webhook(self):
        '''
        Creates or updates Webhook with the specified configuration.

        :return: deserialized Webhook instance state dictionary
        '''
        self.log("Creating / Updating the Webhook instance {0}".format(self.webhook_name))

        try:
            if self.to_do == Actions.Create:
                response = self.mgmt_client.webhooks.create(resource_group_name=self.resource_group,
                                                            registry_name=self.registry_name,
                                                            webhook_name=self.webhook_name,
                                                            webhook_create_parameters=self.parameters)
            else:
                response = self.mgmt_client.webhooks.update(resource_group_name=self.resource_group,
                                                            registry_name=self.registry_name,
                                                            webhook_name=self.webhook_name,
                                                            webhook_update_parameters=self.parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except CloudError as exc:
            self.log('Error attempting to create the Webhook instance.')
            self.fail("Error creating the Webhook instance: {0}".format(str(exc)))
        return create_webhook_dict(response)

    def delete_webhook(self):
        '''
        Deletes specified Webhook instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Webhook instance {0}".format(self.webhook_name))
        try:
            response = self.mgmt_client.webhooks.delete(resource_group_name=self.resource_group,
                                                        registry_name=self.registry_name,
                                                        webhook_name=self.webhook_name)
        except CloudError as e:
            self.log('Error attempting to delete the Webhook instance.')
            self.fail("Error deleting the Webhook instance: {0}".format(str(e)))

        return True

    def get_webhook(self):
        '''
        Gets the properties of the specified Webhook.

        :return: deserialized Webhook instance state dictionary
        '''
        self.log("Checking if the Webhook instance {0} is present".format(self.webhook_name))
        found = False
        try:
            response = self.mgmt_client.webhooks.get(resource_group_name=self.resource_group,
                                                     registry_name=self.registry_name,
                                                     webhook_name=self.webhook_name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Webhook instance : {0} found".format(response.name))
        except CloudError as e:
            self.log('Did not find the Webhook instance.')
        if found is True:
            return response.as_dict()

        return False


def main():
    """Main execution"""
    AzureRMWebhooks()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: openssl_publickey
short_description: Generate an OpenSSL public key from its private key.
description:
    - This module allows one to (re)generate OpenSSL public keys from their private keys.
    - Keys are generated in PEM or OpenSSH format.
    - "The module can use the cryptography Python library, or the pyOpenSSL Python
      library. By default, it tries to detect which one is available. This can be
      overridden with the I(select_crypto_backend) option. When I(format) is C(OpenSSH),
      the C(cryptography) backend has to be used. Please note that the PyOpenSSL backend
      was deprecated in Ansible 2.9 and will be removed in community.crypto 2.0.0."
requirements:
    - Either cryptography >= 1.2.3 (older versions might work as well)
    - Or pyOpenSSL >= 16.0.0
    - Needs cryptography >= 1.4 if I(format) is C(OpenSSH)
author:
    - Yanis Guenane (@Spredzy)
    - Felix Fontein (@felixfontein)
options:
    state:
        description:
            - Whether the public key should exist or not, taking action if the state is different from what is stated.
        type: str
        default: present
        choices: [ absent, present ]
    force:
        description:
            - Should the key be regenerated even it it already exists.
        type: bool
        default: no
    format:
        description:
            - The format of the public key.
        type: str
        default: PEM
        choices: [ OpenSSH, PEM ]
    path:
        description:
            - Name of the file in which the generated TLS/SSL public key will be written.
        type: path
        required: true
    privatekey_path:
        description:
            - Path to the TLS/SSL private key from which to generate the public key.
            - Either I(privatekey_path) or I(privatekey_content) must be specified, but not both.
              If I(state) is C(present), one of them is required.
        type: path
    privatekey_content:
        description:
            - The content of the TLS/SSL private key from which to generate the public key.
            - Either I(privatekey_path) or I(privatekey_content) must be specified, but not both.
              If I(state) is C(present), one of them is required.
        type: str
        version_added: '1.0.0'
    privatekey_passphrase:
        description:
            - The passphrase for the private key.
        type: str
    backup:
        description:
            - Create a backup file including a timestamp so you can get the original
              public key back if you overwrote it with a different one by accident.
        type: bool
        default: no
    select_crypto_backend:
        description:
            - Determines which crypto backend to use.
            - The default choice is C(auto), which tries to use C(cryptography) if available, and falls back to C(pyopenssl).
            - If set to C(pyopenssl), will try to use the L(pyOpenSSL,https://pypi.org/project/pyOpenSSL/) library.
            - If set to C(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
        type: str
        default: auto
        choices: [ auto, cryptography, pyopenssl ]
    return_content:
        description:
            - If set to C(yes), will return the (current or generated) public key's content as I(publickey).
        type: bool
        default: no
        version_added: '1.0.0'
extends_documentation_fragment:
- files
seealso:
- module: community.crypto.x509_certificate
- module: community.crypto.openssl_csr
- module: community.crypto.openssl_dhparam
- module: community.crypto.openssl_pkcs12
- module: community.crypto.openssl_privatekey
'''

EXAMPLES = r'''
- name: Generate an OpenSSL public key in PEM format
  community.crypto.openssl_publickey:
    path: /etc/ssl/public/ansible.com.pem
    privatekey_path: /etc/ssl/private/ansible.com.pem

- name: Generate an OpenSSL public key in PEM format from an inline key
  community.crypto.openssl_publickey:
    path: /etc/ssl/public/ansible.com.pem
    privatekey_content: "{{ private_key_content }}"

- name: Generate an OpenSSL public key in OpenSSH v2 format
  community.crypto.openssl_publickey:
    path: /etc/ssl/public/ansible.com.pem
    privatekey_path: /etc/ssl/private/ansible.com.pem
    format: OpenSSH

- name: Generate an OpenSSL public key with a passphrase protected private key
  community.crypto.openssl_publickey:
    path: /etc/ssl/public/ansible.com.pem
    privatekey_path: /etc/ssl/private/ansible.com.pem
    privatekey_passphrase: ansible

- name: Force regenerate an OpenSSL public key if it already exists
  community.crypto.openssl_publickey:
    path: /etc/ssl/public/ansible.com.pem
    privatekey_path: /etc/ssl/private/ansible.com.pem
    force: yes

- name: Remove an OpenSSL public key
  community.crypto.openssl_publickey:
    path: /etc/ssl/public/ansible.com.pem
    state: absent
'''

RETURN = r'''
privatekey:
    description:
    - Path to the TLS/SSL private key the public key was generated from.
    - Will be C(none) if the private key has been provided in I(privatekey_content).
    returned: changed or success
    type: str
    sample: /etc/ssl/private/ansible.com.pem
format:
    description: The format of the public key (PEM, OpenSSH, ...).
    returned: changed or success
    type: str
    sample: PEM
filename:
    description: Path to the generated TLS/SSL public key file.
    returned: changed or success
    type: str
    sample: /etc/ssl/public/ansible.com.pem
fingerprint:
    description:
    - The fingerprint of the public key. Fingerprint will be generated for each hashlib.algorithms available.
    - Requires PyOpenSSL >= 16.0 for meaningful output.
    returned: changed or success
    type: dict
    sample:
      md5: "84:75:71:72:8d:04:b5:6c:4d:37:6d:66:83:f5:4c:29"
      sha1: "51:cc:7c:68:5d:eb:41:43:88:7e:1a:ae:c7:f8:24:72:ee:71:f6:10"
      sha224: "b1:19:a6:6c:14:ac:33:1d:ed:18:50:d3:06:5c:b2:32:91:f1:f1:52:8c:cb:d5:75:e9:f5:9b:46"
      sha256: "41:ab:c7:cb:d5:5f:30:60:46:99:ac:d4:00:70:cf:a1:76:4f:24:5d:10:24:57:5d:51:6e:09:97:df:2f:de:c7"
      sha384: "85:39:50:4e:de:d9:19:33:40:70:ae:10:ab:59:24:19:51:c3:a2:e4:0b:1c:b1:6e:dd:b3:0c:d9:9e:6a:46:af:da:18:f8:ef:ae:2e:c0:9a:75:2c:9b:b3:0f:3a:5f:3d"
      sha512: "fd:ed:5e:39:48:5f:9f:fe:7f:25:06:3f:79:08:cd:ee:a5:e7:b3:3d:13:82:87:1f:84:e1:f5:c7:28:77:53:94:86:56:38:69:f0:d9:35:22:01:1e:a6:60:...:0f:9b"
backup_file:
    description: Name of backup file created.
    returned: changed and if I(backup) is C(yes)
    type: str
    sample: /path/to/publickey.pem.2019-03-09@11:22~
publickey:
    description: The (current or generated) public key's content.
    returned: if I(state) is C(present) and I(return_content) is C(yes)
    type: str
    version_added: '1.0.0'
'''

import os
import traceback

from distutils.version import LooseVersion

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils._text import to_native

from ansible_collections.community.crypto.plugins.module_utils.io import (
    load_file_if_exists,
    write_file,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.basic import (
    OpenSSLObjectError,
    OpenSSLBadPassphraseError,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.support import (
    OpenSSLObject,
    load_privatekey,
    get_fingerprint,
)

MINIMAL_PYOPENSSL_VERSION = '16.0.0'
MINIMAL_CRYPTOGRAPHY_VERSION = '1.2.3'
MINIMAL_CRYPTOGRAPHY_VERSION_OPENSSH = '1.4'

PYOPENSSL_IMP_ERR = None
try:
    import OpenSSL
    from OpenSSL import crypto
    PYOPENSSL_VERSION = LooseVersion(OpenSSL.__version__)
except ImportError:
    PYOPENSSL_IMP_ERR = traceback.format_exc()
    PYOPENSSL_FOUND = False
else:
    PYOPENSSL_FOUND = True

CRYPTOGRAPHY_IMP_ERR = None
try:
    import cryptography
    from cryptography.hazmat.backends import default_backend
    from cryptography.hazmat.primitives import serialization as crypto_serialization
    CRYPTOGRAPHY_VERSION = LooseVersion(cryptography.__version__)
except ImportError:
    CRYPTOGRAPHY_IMP_ERR = traceback.format_exc()
    CRYPTOGRAPHY_FOUND = False
else:
    CRYPTOGRAPHY_FOUND = True


class PublicKeyError(OpenSSLObjectError):
    pass


class PublicKey(OpenSSLObject):

    def __init__(self, module, backend):
        super(PublicKey, self).__init__(
            module.params['path'],
            module.params['state'],
            module.params['force'],
            module.check_mode
        )
        self.format = module.params['format']
        self.privatekey_path = module.params['privatekey_path']
        self.privatekey_content = module.params['privatekey_content']
        if self.privatekey_content is not None:
            self.privatekey_content = self.privatekey_content.encode('utf-8')
        self.privatekey_passphrase = module.params['privatekey_passphrase']
        self.privatekey = None
        self.publickey_bytes = None
        self.return_content = module.params['return_content']
        self.fingerprint = {}
        self.backend = backend

        self.backup = module.params['backup']
        self.backup_file = None

    def _create_publickey(self, module):
        self.privatekey = load_privatekey(
            path=self.privatekey_path,
            content=self.privatekey_content,
            passphrase=self.privatekey_passphrase,
            backend=self.backend
        )
        if self.backend == 'cryptography':
            if self.format == 'OpenSSH':
                return self.privatekey.public_key().public_bytes(
                    crypto_serialization.Encoding.OpenSSH,
                    crypto_serialization.PublicFormat.OpenSSH
                )
            else:
                return self.privatekey.public_key().public_bytes(
                    crypto_serialization.Encoding.PEM,
                    crypto_serialization.PublicFormat.SubjectPublicKeyInfo
                )
        else:
            try:
                return crypto.dump_publickey(crypto.FILETYPE_PEM, self.privatekey)
            except AttributeError as dummy:
                raise PublicKeyError('You need to have PyOpenSSL>=16.0.0 to generate public keys')

    def generate(self, module):
        """Generate the public key."""

        if self.privatekey_content is None and not os.path.exists(self.privatekey_path):
            raise PublicKeyError(
                'The private key %s does not exist' % self.privatekey_path
            )

        if not self.check(module, perms_required=False) or self.force:
            try:
                publickey_content = self._create_publickey(module)
                if self.return_content:
                    self.publickey_bytes = publickey_content

                if self.backup:
                    self.backup_file = module.backup_local(self.path)
                write_file(module, publickey_content)

                self.changed = True
            except OpenSSLBadPassphraseError as exc:
                raise PublicKeyError(exc)
            except (IOError, OSError) as exc:
                raise PublicKeyError(exc)

        self.fingerprint = get_fingerprint(
            path=self.privatekey_path,
            content=self.privatekey_content,
            passphrase=self.privatekey_passphrase,
            backend=self.backend,
        )
        file_args = module.load_file_common_arguments(module.params)
        if module.set_fs_attributes_if_different(file_args, False):
            self.changed = True

    def check(self, module, perms_required=True):
        """Ensure the resource is in its desired state."""

        state_and_perms = super(PublicKey, self).check(module, perms_required)

        def _check_privatekey():
            if self.privatekey_content is None and not os.path.exists(self.privatekey_path):
                return False

            try:
                with open(self.path, 'rb') as public_key_fh:
                    publickey_content = public_key_fh.read()
                if self.return_content:
                    self.publickey_bytes = publickey_content
                if self.backend == 'cryptography':
                    if self.format == 'OpenSSH':
                        # Read and dump public key. Makes sure that the comment is stripped off.
                        current_publickey = crypto_serialization.load_ssh_public_key(publickey_content, backend=default_backend())
                        publickey_content = current_publickey.public_bytes(
                            crypto_serialization.Encoding.OpenSSH,
                            crypto_serialization.PublicFormat.OpenSSH
                        )
                    else:
                        current_publickey = crypto_serialization.load_pem_public_key(publickey_content, backend=default_backend())
                        publickey_content = current_publickey.public_bytes(
                            crypto_serialization.Encoding.PEM,
                            crypto_serialization.PublicFormat.SubjectPublicKeyInfo
                        )
                else:
                    publickey_content = crypto.dump_publickey(
                        crypto.FILETYPE_PEM,
                        crypto.load_publickey(crypto.FILETYPE_PEM, publickey_content)
                    )
            except Exception as dummy:
                return False

            try:
                desired_publickey = self._create_publickey(module)
            except OpenSSLBadPassphraseError as exc:
                raise PublicKeyError(exc)

            return publickey_content == desired_publickey

        if not state_and_perms:
            return state_and_perms

        return _check_privatekey()

    def remove(self, module):
        if self.backup:
            self.backup_file = module.backup_local(self.path)
        super(PublicKey, self).remove(module)

    def dump(self):
        """Serialize the object into a dictionary."""

        result = {
            'privatekey': self.privatekey_path,
            'filename': self.path,
            'format': self.format,
            'changed': self.changed,
            'fingerprint': self.fingerprint,
        }
        if self.backup_file:
            result['backup_file'] = self.backup_file
        if self.return_content:
            if self.publickey_bytes is None:
                self.publickey_bytes = load_file_if_exists(self.path, ignore_errors=True)
            result['publickey'] = self.publickey_bytes.decode('utf-8') if self.publickey_bytes else None

        return result


def main():

    module = AnsibleModule(
        argument_spec=dict(
            state=dict(type='str', default='present', choices=['present', 'absent']),
            force=dict(type='bool', default=False),
            path=dict(type='path', required=True),
            privatekey_path=dict(type='path'),
            privatekey_content=dict(type='str'),
            format=dict(type='str', default='PEM', choices=['OpenSSH', 'PEM']),
            privatekey_passphrase=dict(type='str', no_log=True),
            backup=dict(type='bool', default=False),
            select_crypto_backend=dict(type='str', choices=['auto', 'pyopenssl', 'cryptography'], default='auto'),
            return_content=dict(type='bool', default=False),
        ),
        supports_check_mode=True,
        add_file_common_args=True,
        required_if=[('state', 'present', ['privatekey_path', 'privatekey_content'], True)],
        mutually_exclusive=(
            ['privatekey_path', 'privatekey_content'],
        ),
    )

    minimal_cryptography_version = MINIMAL_CRYPTOGRAPHY_VERSION
    if module.params['format'] == 'OpenSSH':
        minimal_cryptography_version = MINIMAL_CRYPTOGRAPHY_VERSION_OPENSSH

    backend = module.params['select_crypto_backend']
    if backend == 'auto':
        # Detection what is possible
        can_use_cryptography = CRYPTOGRAPHY_FOUND and CRYPTOGRAPHY_VERSION >= LooseVersion(minimal_cryptography_version)
        can_use_pyopenssl = PYOPENSSL_FOUND and PYOPENSSL_VERSION >= LooseVersion(MINIMAL_PYOPENSSL_VERSION)

        # Decision
        if can_use_cryptography:
            backend = 'cryptography'
        elif can_use_pyopenssl:
            if module.params['format'] == 'OpenSSH':
                module.fail_json(
                    msg=missing_required_lib('cryptography >= {0}'.format(MINIMAL_CRYPTOGRAPHY_VERSION_OPENSSH)),
                    exception=CRYPTOGRAPHY_IMP_ERR
                )
            backend = 'pyopenssl'

        # Success?
        if backend == 'auto':
            module.fail_json(msg=("Can't detect any of the required Python libraries "
                                  "cryptography (>= {0}) or PyOpenSSL (>= {1})").format(
                                      minimal_cryptography_version,
                                      MINIMAL_PYOPENSSL_VERSION))

    if module.params['format'] == 'OpenSSH' and backend != 'cryptography':
        module.fail_json(msg="Format OpenSSH requires the cryptography backend.")

    if backend == 'pyopenssl':
        if not PYOPENSSL_FOUND:
            module.fail_json(msg=missing_required_lib('pyOpenSSL >= {0}'.format(MINIMAL_PYOPENSSL_VERSION)),
                             exception=PYOPENSSL_IMP_ERR)
        module.deprecate('The module is using the PyOpenSSL backend. This backend has been deprecated',
                         version='2.0.0', collection_name='community.crypto')
    elif backend == 'cryptography':
        if not CRYPTOGRAPHY_FOUND:
            module.fail_json(msg=missing_required_lib('cryptography >= {0}'.format(minimal_cryptography_version)),
                             exception=CRYPTOGRAPHY_IMP_ERR)

    base_dir = os.path.dirname(module.params['path']) or '.'
    if not os.path.isdir(base_dir):
        module.fail_json(
            name=base_dir,
            msg="The directory '%s' does not exist or the file is not a directory" % base_dir
        )

    try:
        public_key = PublicKey(module, backend)

        if public_key.state == 'present':
            if module.check_mode:
                result = public_key.dump()
                result['changed'] = module.params['force'] or not public_key.check(module)
                module.exit_json(**result)

            public_key.generate(module)
        else:
            if module.check_mode:
                result = public_key.dump()
                result['changed'] = os.path.exists(module.params['path'])
                module.exit_json(**result)

            public_key.remove(module)

        result = public_key.dump()
        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=to_native(exc))


if __name__ == '__main__':
    main()



#
# (c) 2019 Lenovo.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.cnos import cnos_vrf
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .cnos_module import TestCnosModule, load_fixture


class TestCnosVrfModule(TestCnosModule):

    module = cnos_vrf

    def setUp(self):
        super(TestCnosVrfModule, self).setUp()

        self.mock_load_config = patch('ansible_collections.community.network.plugins.modules.network.cnos.cnos_vrf.load_config')
        self.load_config = self.mock_load_config.start()

        self.mock_run_commands = patch('ansible_collections.community.network.plugins.modules.network.cnos.cnos_vrf.run_commands')
        self.run_commands = self.mock_run_commands.start()

        self._patch_is_switchport = patch(
            'ansible_collections.community.network.plugins.modules.network.cnos.cnos_vrf.is_switchport'
        )
        self._is_switchport = self._patch_is_switchport.start()

    def tearDown(self):
        super(TestCnosVrfModule, self).tearDown()
        self.mock_load_config.stop()
        self.mock_run_commands.stop()
        self._patch_is_switchport.stop()

    def load_fixtures(self, commands=None):
        config_file = 'cnos_vrf_config.cfg'
        self.load_config.return_value = load_fixture(config_file)
        self.run_commands.return_value = load_fixture(config_file)
        self._is_switchport.return_value = False

    def test_cnos_vrf_present(self):
        set_module_args(dict(name='test1', state='present'))
        self.execute_module(changed=True, commands=['vrf context test1'])

    def test_cnos_vrf_present_management(self):
        set_module_args(dict(name='management', state='present'))
        self.execute_module(changed=True, commands=['vrf context management'])

    def test_cnos_vrf_absent_management(self):
        set_module_args(dict(name='management', state='absent'))
        result = self.execute_module(failed=True)
        self.assertEqual(result['msg'], 'Management VRF context cannot be deleted')

    def test_cnos_vrf_absent_no_change(self):
        set_module_args(dict(name='test1', state='absent'))
        self.execute_module(changed=False, commands=[])

    def test_cnos_vrf_default(self):
        set_module_args(dict(name='default', state='present'))
        result = self.execute_module(failed=True)
        self.assertEqual(result['msg'], 'VRF context default is reserved')



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: ce_acl_advance
short_description: Manages advanced ACL configuration on HUAWEI CloudEngine switches.
description:
    - Manages advanced ACL configurations on HUAWEI CloudEngine switches.
author:
    - wangdezhuang (@QijunPan)
notes:
  - This module requires the netconf system service be enabled on the remote device being managed.
  - Recommended connection is C(netconf).
  - This module also works with C(local) connections for legacy playbooks.
options:
    state:
        description:
            - Specify desired state of the resource.
        required: false
        default: present
        choices: ['present','absent','delete_acl']
    acl_name:
        description:
            - ACL number or name.
              For a numbered rule group, the value ranging from 3000 to 3999 indicates a advance ACL.
              For a named rule group, the value is a string of 1 to 32 case-sensitive characters starting
              with a letter, spaces not supported.
        required: true
    acl_num:
        description:
            - ACL number.
              The value is an integer ranging from 3000 to 3999.
    acl_step:
        description:
            - ACL step.
              The value is an integer ranging from 1 to 20. The default value is 5.
    acl_description:
        description:
            - ACL description.
              The value is a string of 1 to 127 characters.
    rule_name:
        description:
            - Name of a basic ACL rule.
              The value is a string of 1 to 32 characters.
    rule_id:
        description:
            - ID of a basic ACL rule in configuration mode.
              The value is an integer ranging from 0 to 4294967294.
    rule_action:
        description:
            - Matching mode of basic ACL rules.
        choices: ['permit','deny']
    protocol:
        description:
            - Protocol type.
        choices: ['ip', 'icmp', 'igmp', 'ipinip', 'tcp', 'udp', 'gre', 'ospf']
    source_ip:
        description:
            - Source IP address.
              The value is a string of 0 to 255 characters.The default value is 0.0.0.0.
              The value is in dotted decimal notation.
    src_mask:
        description:
            - Source IP address mask.
              The value is an integer ranging from 1 to 32.
    src_pool_name:
        description:
            - Name of a source pool.
              The value is a string of 1 to 32 characters.
    dest_ip:
        description:
            - Destination IP address.
              The value is a string of 0 to 255 characters.The default value is 0.0.0.0.
              The value is in dotted decimal notation.
    dest_mask:
        description:
            - Destination IP address mask.
              The value is an integer ranging from 1 to 32.
    dest_pool_name:
        description:
            - Name of a destination pool.
              The value is a string of 1 to 32 characters.
    src_port_op:
        description:
            - Range type of the source port.
        choices: ['lt','eq', 'gt', 'range']
    src_port_begin:
        description:
            - Start port number of the source port.
              The value is an integer ranging from 0 to 65535.
    src_port_end:
        description:
            - End port number of the source port.
              The value is an integer ranging from 0 to 65535.
    src_port_pool_name:
        description:
            - Name of a source port pool.
              The value is a string of 1 to 32 characters.
    dest_port_op:
        description:
            - Range type of the destination port.
        choices: ['lt','eq', 'gt', 'range']
    dest_port_begin:
        description:
            - Start port number of the destination port.
              The value is an integer ranging from 0 to 65535.
    dest_port_end:
        description:
            - End port number of the destination port.
              The value is an integer ranging from 0 to 65535.
    dest_port_pool_name:
        description:
            - Name of a destination port pool.
              The value is a string of 1 to 32 characters.
    frag_type:
        description:
            - Type of packet fragmentation.
        choices: ['fragment', 'clear_fragment']
    precedence:
        description:
            - Data packets can be filtered based on the priority field.
              The value is an integer ranging from 0 to 7.
    tos:
        description:
            - ToS value on which data packet filtering is based.
              The value is an integer ranging from 0 to 15.
    dscp:
        description:
            - Differentiated Services Code Point.
              The value is an integer ranging from 0 to 63.
    icmp_name:
        description:
            - ICMP name.
        choices: ['unconfiged', 'echo', 'echo-reply', 'fragmentneed-DFset', 'host-redirect',
                  'host-tos-redirect', 'host-unreachable', 'information-reply', 'information-request',
                  'net-redirect', 'net-tos-redirect', 'net-unreachable', 'parameter-problem',
                  'port-unreachable', 'protocol-unreachable', 'reassembly-timeout', 'source-quench',
                  'source-route-failed', 'timestamp-reply', 'timestamp-request', 'ttl-exceeded',
                  'address-mask-reply', 'address-mask-request', 'custom']
    icmp_type:
        description:
            - ICMP type. This parameter is available only when the packet protocol is ICMP.
              The value is an integer ranging from 0 to 255.
    icmp_code:
        description:
            - ICMP message code. Data packets can be filtered based on the ICMP message code.
              The value is an integer ranging from 0 to 255.
    ttl_expired:
        description:
            - Whether TTL Expired is matched, with the TTL value of 1.
        type: bool
        default: 'no'
    vrf_name:
        description:
            - VPN instance name.
              The value is a string of 1 to 31 characters.The default value is _public_.
    syn_flag:
        description:
            - TCP flag value.
              The value is an integer ranging from 0 to 63.
    tcp_flag_mask:
        description:
            - TCP flag mask value.
              The value is an integer ranging from 0 to 63.
    established:
        description:
            - Match established connections.
        type: bool
        default: 'no'
    time_range:
        description:
            - Name of a time range in which an ACL rule takes effect.
    rule_description:
        description:
            - Description about an ACL rule.
    igmp_type:
        description:
            - Internet Group Management Protocol.
        choices: ['host-query', 'mrouter-adver', 'mrouter-solic', 'mrouter-termi', 'mtrace-resp', 'mtrace-route',
                  'v1host-report', 'v2host-report', 'v2leave-group', 'v3host-report']
    log_flag:
        description:
            - Flag of logging matched data packets.
        type: bool
        default: 'no'
'''

EXAMPLES = '''

- name: CloudEngine advance acl test
  hosts: cloudengine
  connection: local
  gather_facts: no
  vars:
    cli:
      host: "{{ inventory_hostname }}"
      port: "{{ ansible_ssh_port }}"
      username: "{{ username }}"
      password: "{{ password }}"
      transport: cli

  tasks:

  - name: "Config ACL"
    community.network.ce_acl_advance:
      state: present
      acl_name: 3200
      provider: "{{ cli }}"

  - name: "Undo ACL"
    community.network.ce_acl_advance:
      state: delete_acl
      acl_name: 3200
      provider: "{{ cli }}"

  - name: "Config ACL advance rule"
    community.network.ce_acl_advance:
      state: present
      acl_name: test
      rule_name: test_rule
      rule_id: 111
      rule_action: permit
      protocol: tcp
      source_ip: 10.10.10.10
      src_mask: 24
      frag_type: fragment
      provider: "{{ cli }}"

  - name: "Undo ACL advance rule"
    community.network.ce_acl_advance:
      state: absent
      acl_name: test
      rule_name: test_rule
      rule_id: 111
      rule_action: permit
      protocol: tcp
      source_ip: 10.10.10.10
      src_mask: 24
      frag_type: fragment
      provider: "{{ cli }}"
'''

RETURN = '''
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"acl_name": "test", "state": "delete_acl"}
existing:
    description: k/v pairs of existing aaa server
    returned: always
    type: dict
    sample: {"aclNumOrName": "test", "aclType": "Advance"}
end_state:
    description: k/v pairs of aaa params after module execution
    returned: always
    type: dict
    sample: {}
updates:
    description: command sent to the device
    returned: always
    type: list
    sample: ["undo acl name test"]
'''

from xml.etree import ElementTree
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import get_nc_config, set_nc_config, ce_argument_spec, check_ip_addr


# get acl
CE_GET_ACL_HEADER = """
    <filter type="subtree">
      <acl xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <aclGroups>
          <aclGroup>
            <aclNumOrName></aclNumOrName>
"""
CE_GET_ACL_TAIL = """
          </aclGroup>
        </aclGroups>
      </acl>
    </filter>
"""
# merge acl
CE_MERGE_ACL_HEADER = """
    <config>
      <acl xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <aclGroups>
          <aclGroup operation="merge">
            <aclNumOrName>%s</aclNumOrName>
"""
CE_MERGE_ACL_TAIL = """
          </aclGroup>
        </aclGroups>
      </acl>
    </config>
"""
# delete acl
CE_DELETE_ACL_HEADER = """
    <config>
      <acl xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <aclGroups>
          <aclGroup operation="delete">
            <aclNumOrName>%s</aclNumOrName>
"""
CE_DELETE_ACL_TAIL = """
          </aclGroup>
        </aclGroups>
      </acl>
    </config>
"""

# get acl advance rule
CE_GET_ACL_ADVANCE_RULE_HEADER = """
    <filter type="subtree">
      <acl xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <aclGroups>
          <aclGroup>
            <aclNumOrName>%s</aclNumOrName>
            <aclRuleAdv4s>
              <aclRuleAdv4>
                <aclRuleName></aclRuleName>
"""
CE_GET_ACL_ADVANCE_RULE_TAIL = """
              </aclRuleAdv4>
            </aclRuleAdv4s>
          </aclGroup>
        </aclGroups>
      </acl>
    </filter>
"""
# merge acl advance rule
CE_MERGE_ACL_ADVANCE_RULE_HEADER = """
    <config>
      <acl xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <aclGroups>
          <aclGroup>
            <aclNumOrName>%s</aclNumOrName>
            <aclRuleAdv4s>
              <aclRuleAdv4 operation="merge">
                <aclRuleName>%s</aclRuleName>
"""
CE_MERGE_ACL_ADVANCE_RULE_TAIL = """
              </aclRuleAdv4>
            </aclRuleAdv4s>
          </aclGroup>
        </aclGroups>
      </acl>
    </config>
"""
# delete acl advance rule
CE_DELETE_ACL_ADVANCE_RULE_HEADER = """
    <config>
      <acl xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <aclGroups>
          <aclGroup>
            <aclNumOrName>%s</aclNumOrName>
            <aclRuleAdv4s>
              <aclRuleAdv4 operation="delete">
                <aclRuleName>%s</aclRuleName>
"""
CE_DELETE_ACL_ADVANCE_RULE_TAIL = """
              </aclRuleAdv4>
            </aclRuleAdv4s>
          </aclGroup>
        </aclGroups>
      </acl>
    </config>
"""


PROTOCOL_NUM = {"ip": "0",
                "icmp": "1",
                "igmp": "2",
                "ipinip": "4",
                "tcp": "6",
                "udp": "17",
                "gre": "47",
                "ospf": "89"}

IGMP_TYPE_NUM = {"host-query": "17",
                 "mrouter-adver": "48",
                 "mrouter-solic": "49",
                 "mrouter-termi": "50",
                 "mtrace-resp": "30",
                 "mtrace-route": "31",
                 "v1host-report": "18",
                 "v2host-report": "22",
                 "v2leave-group": "23",
                 "v3host-report": "34"}


def get_wildcard_mask(mask):
    """ convert mask length to ip address wildcard mask, i.e. 24 to 0.0.0.255 """

    mask_int = ["255"] * 4
    value = int(mask)

    if value > 32:
        return None
    if value < 8:
        mask_int[0] = str(int(~(0xFF << (8 - value % 8)) & 0xFF))
    if value >= 8:
        mask_int[0] = '0'
        mask_int[1] = str(int(~(0xFF << (16 - (value % 16))) & 0xFF))
    if value >= 16:
        mask_int[1] = '0'
        mask_int[2] = str(int(~(0xFF << (24 - (value % 24))) & 0xFF))
    if value >= 24:
        mask_int[2] = '0'
        mask_int[3] = str(int(~(0xFF << (32 - (value % 32))) & 0xFF))
    if value == 32:
        mask_int[3] = '0'

    return '.'.join(mask_int)


class AdvanceAcl(object):
    """ Manages advance acl configuration """

    def __init__(self, **kwargs):
        """ Class init """

        # argument spec
        argument_spec = kwargs["argument_spec"]
        self.spec = argument_spec
        self.module = AnsibleModule(argument_spec=self.spec, supports_check_mode=True)

        # module args
        self.state = self.module.params['state']
        self.acl_name = self.module.params['acl_name'] or None
        self.acl_num = self.module.params['acl_num'] or None
        self.acl_type = None
        self.acl_step = self.module.params['acl_step'] or None
        self.acl_description = self.module.params['acl_description'] or None
        self.rule_name = self.module.params['rule_name'] or None
        self.rule_id = self.module.params['rule_id'] or None
        self.rule_action = self.module.params['rule_action'] or None
        self.protocol = self.module.params['protocol'] or None
        self.protocol_num = None
        self.source_ip = self.module.params['source_ip'] or None
        self.src_mask = self.module.params['src_mask'] or None
        self.src_wild = None
        self.src_pool_name = self.module.params['src_pool_name'] or None
        self.dest_ip = self.module.params['dest_ip'] or None
        self.dest_mask = self.module.params['dest_mask'] or None
        self.dest_wild = None
        self.dest_pool_name = self.module.params['dest_pool_name'] or None
        self.src_port_op = self.module.params['src_port_op'] or None
        self.src_port_begin = self.module.params['src_port_begin'] or None
        self.src_port_end = self.module.params['src_port_end'] or None
        self.src_port_pool_name = self.module.params[
            'src_port_pool_name'] or None
        self.dest_port_op = self.module.params['dest_port_op'] or None
        self.dest_port_begin = self.module.params['dest_port_begin'] or None
        self.dest_port_end = self.module.params['dest_port_end'] or None
        self.dest_port_pool_name = self.module.params[
            'dest_port_pool_name'] or None
        self.frag_type = self.module.params['frag_type'] or None
        self.precedence = self.module.params['precedence'] or None
        self.tos = self.module.params['tos'] or None
        self.dscp = self.module.params['dscp'] or None
        self.icmp_name = self.module.params['icmp_name'] or None
        self.icmp_type = self.module.params['icmp_type'] or None
        self.icmp_code = self.module.params['icmp_code'] or None
        self.ttl_expired = self.module.params['ttl_expired']
        self.vrf_name = self.module.params['vrf_name'] or None
        self.syn_flag = self.module.params['syn_flag'] or None
        self.tcp_flag_mask = self.module.params['tcp_flag_mask'] or None
        self.established = self.module.params['established']
        self.time_range = self.module.params['time_range'] or None
        self.rule_description = self.module.params['rule_description'] or None
        self.igmp_type = self.module.params['igmp_type'] or None
        self.igmp_type_num = None
        self.log_flag = self.module.params['log_flag']

        self.precedence_name = dict()
        self.precedence_name["0"] = "routine"
        self.precedence_name["1"] = "priority"
        self.precedence_name["2"] = "immediate"
        self.precedence_name["3"] = "flash"
        self.precedence_name["4"] = "flash-override"
        self.precedence_name["5"] = "critical"
        self.precedence_name["6"] = "internet"
        self.precedence_name["7"] = "network"

        # cur config
        self.cur_acl_cfg = dict()
        self.cur_advance_rule_cfg = dict()

        # state
        self.changed = False
        self.updates_cmd = list()
        self.results = dict()
        self.proposed = dict()
        self.existing = dict()
        self.end_state = dict()

    def netconf_get_config(self, conf_str):
        """ Get configure by netconf """

        xml_str = get_nc_config(self.module, conf_str)

        return xml_str

    def netconf_set_config(self, conf_str):
        """ Set configure by netconf """

        xml_str = set_nc_config(self.module, conf_str)

        return xml_str

    def get_protocol_num(self):
        """ Get protocol num by name """

        if self.protocol:
            self.protocol_num = PROTOCOL_NUM.get(self.protocol)

    def get_igmp_type_num(self):
        """ Get igmp type num by type """

        if self.igmp_type:
            self.igmp_type_num = IGMP_TYPE_NUM.get(self.igmp_type)

    def check_acl_args(self):
        """ Check acl invalid args """

        need_cfg = False
        find_flag = False
        self.cur_acl_cfg["acl_info"] = []

        if self.acl_name:

            if self.acl_name.isdigit():
                if int(self.acl_name) < 3000 or int(self.acl_name) > 3999:
                    self.module.fail_json(
                        msg='Error: The value of acl_name is out of [3000-3999] for advance ACL.')

                if self.acl_num:
                    self.module.fail_json(
                        msg='Error: The acl_name is digit, so should not input acl_num at the same time.')
            else:

                self.acl_type = "Advance"

                if len(self.acl_name) < 1 or len(self.acl_name) > 32:
                    self.module.fail_json(
                        msg='Error: The len of acl_name is out of [1 - 32].')

                if self.state == "present":
                    if not self.acl_num and not self.acl_type and not self.rule_name:
                        self.module.fail_json(
                            msg='Error: Please input acl_num or acl_type when config ACL.')

            if self.acl_num:
                if self.acl_num.isdigit():
                    if int(self.acl_num) < 3000 or int(self.acl_num) > 3999:
                        self.module.fail_json(
                            msg='Error: The value of acl_name is out of [3000-3999] for advance ACL.')
                else:
                    self.module.fail_json(
                        msg='Error: The acl_num is not digit.')

            if self.acl_step:
                if self.acl_step.isdigit():
                    if int(self.acl_step) < 1 or int(self.acl_step) > 20:
                        self.module.fail_json(
                            msg='Error: The value of acl_step is out of [1 - 20].')
                else:
                    self.module.fail_json(
                        msg='Error: The acl_step is not digit.')

            if self.acl_description:
                if len(self.acl_description) < 1 or len(self.acl_description) > 127:
                    self.module.fail_json(
                        msg='Error: The len of acl_description is out of [1 - 127].')

            conf_str = CE_GET_ACL_HEADER

            if self.acl_type:
                conf_str += "<aclType></aclType>"
            if self.acl_num or self.acl_name.isdigit():
                conf_str += "<aclNumber></aclNumber>"
            if self.acl_step:
                conf_str += "<aclStep></aclStep>"
            if self.acl_description:
                conf_str += "<aclDescription></aclDescription>"

            conf_str += CE_GET_ACL_TAIL
            recv_xml = self.netconf_get_config(conf_str=conf_str)

            if "<data/>" in recv_xml:
                find_flag = False

            else:
                xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                    replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                    replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

                root = ElementTree.fromstring(xml_str)

                # parse acl
                acl_info = root.findall(
                    "acl/aclGroups/aclGroup")
                if acl_info:
                    for tmp in acl_info:
                        tmp_dict = dict()
                        for site in tmp:
                            if site.tag in ["aclNumOrName", "aclType", "aclNumber", "aclStep", "aclDescription"]:
                                tmp_dict[site.tag] = site.text

                        self.cur_acl_cfg["acl_info"].append(tmp_dict)

                if self.cur_acl_cfg["acl_info"]:
                    find_list = list()
                    for tmp in self.cur_acl_cfg["acl_info"]:
                        cur_cfg_dict = dict()
                        exist_cfg_dict = dict()

                        if self.acl_name:
                            if self.acl_name.isdigit() and tmp.get("aclNumber"):
                                cur_cfg_dict["aclNumber"] = self.acl_name
                                exist_cfg_dict["aclNumber"] = tmp.get("aclNumber")
                            else:
                                cur_cfg_dict["aclNumOrName"] = self.acl_name
                                exist_cfg_dict["aclNumOrName"] = tmp.get("aclNumOrName")
                        if self.acl_type:
                            cur_cfg_dict["aclType"] = self.acl_type
                            exist_cfg_dict["aclType"] = tmp.get("aclType")
                        if self.acl_num:
                            cur_cfg_dict["aclNumber"] = self.acl_num
                            exist_cfg_dict["aclNumber"] = tmp.get("aclNumber")
                        if self.acl_step:
                            cur_cfg_dict["aclStep"] = self.acl_step
                            exist_cfg_dict["aclStep"] = tmp.get("aclStep")
                        if self.acl_description:
                            cur_cfg_dict["aclDescription"] = self.acl_description
                            exist_cfg_dict["aclDescription"] = tmp.get("aclDescription")

                        if cur_cfg_dict == exist_cfg_dict:
                            find_bool = True
                        else:
                            find_bool = False
                        find_list.append(find_bool)
                    for mem in find_list:
                        if mem:
                            find_flag = True
                            break
                        else:
                            find_flag = False
                else:
                    find_flag = False

        if self.state == "present":
            need_cfg = bool(not find_flag)
        elif self.state == "delete_acl":
            need_cfg = bool(find_flag)
        else:
            need_cfg = False

        self.cur_acl_cfg["need_cfg"] = need_cfg

    def check_advance_rule_args(self):
        """ Check advance rule invalid args """

        need_cfg = False
        find_flag = False
        self.cur_advance_rule_cfg["adv_rule_info"] = []

        if self.acl_name:

            if self.state == "absent":
                if not self.rule_name:
                    self.module.fail_json(
                        msg='Error: Please input rule_name when state is absent.')

            # config rule
            if self.rule_name:
                if len(self.rule_name) < 1 or len(self.rule_name) > 32:
                    self.module.fail_json(
                        msg='Error: The len of rule_name is out of [1 - 32].')

                if self.state != "delete_acl" and not self.rule_id:
                    self.module.fail_json(
                        msg='Error: Please input rule_id.')

                if self.rule_id:
                    if self.rule_id.isdigit():
                        if int(self.rule_id) < 0 or int(self.rule_id) > 4294967294:
                            self.module.fail_json(
                                msg='Error: The value of rule_id is out of [0 - 4294967294].')
                    else:
                        self.module.fail_json(
                            msg='Error: The rule_id is not digit.')

                if self.rule_action and not self.protocol:
                    self.module.fail_json(
                        msg='Error: The rule_action and the protocol must input at the same time.')

                if not self.rule_action and self.protocol:
                    self.module.fail_json(
                        msg='Error: The rule_action and the protocol must input at the same time.')

                if self.protocol:
                    self.get_protocol_num()

                if self.source_ip:
                    if not check_ip_addr(self.source_ip):
                        self.module.fail_json(
                            msg='Error: The source_ip %s is invalid.' % self.source_ip)
                    if not self.src_mask:
                        self.module.fail_json(
                            msg='Error: Please input src_mask.')

                if self.src_mask:
                    if self.src_mask.isdigit():
                        if int(self.src_mask) < 1 or int(self.src_mask) > 32:
                            self.module.fail_json(
                                msg='Error: The value of src_mask is out of [1 - 32].')
                        self.src_wild = get_wildcard_mask(self.src_mask)
                    else:
                        self.module.fail_json(
                            msg='Error: The src_mask is not digit.')

                if self.src_pool_name:
                    if len(self.src_pool_name) < 1 or len(self.src_pool_name) > 32:
                        self.module.fail_json(
                            msg='Error: The len of src_pool_name is out of [1 - 32].')

                if self.dest_ip:
                    if not check_ip_addr(self.dest_ip):
                        self.module.fail_json(
                            msg='Error: The dest_ip %s is invalid.' % self.dest_ip)
                    if not self.dest_mask:
                        self.module.fail_json(
                            msg='Error: Please input dest_mask.')

                if self.dest_mask:
                    if self.dest_mask.isdigit():
                        if int(self.dest_mask) < 1 or int(self.dest_mask) > 32:
                            self.module.fail_json(
                                msg='Error: The value of dest_mask is out of [1 - 32].')
                        self.dest_wild = get_wildcard_mask(self.dest_mask)
                    else:
                        self.module.fail_json(
                            msg='Error: The dest_mask is not digit.')

                if self.dest_pool_name:
                    if len(self.dest_pool_name) < 1 or len(self.dest_pool_name) > 32:
                        self.module.fail_json(
                            msg='Error: The len of dest_pool_name is out of [1 - 32].')

                if self.src_port_op:
                    if self.src_port_op == "lt":
                        if not self.src_port_end:
                            self.module.fail_json(
                                msg='Error: The src_port_end must input.')
                        if self.src_port_begin:
                            self.module.fail_json(
                                msg='Error: The src_port_begin should not input.')
                    if self.src_port_op == "eq" or self.src_port_op == "gt":
                        if not self.src_port_begin:
                            self.module.fail_json(
                                msg='Error: The src_port_begin must input.')
                        if self.src_port_end:
                            self.module.fail_json(
                                msg='Error: The src_port_end should not input.')
                    if self.src_port_op == "range":
                        if not self.src_port_begin or not self.src_port_end:
                            self.module.fail_json(
                                msg='Error: The src_port_begin and src_port_end must input.')

                if self.src_port_begin:
                    if self.src_port_begin.isdigit():
                        if int(self.src_port_begin) < 0 or int(self.src_port_begin) > 65535:
                            self.module.fail_json(
                                msg='Error: The value of src_port_begin is out of [0 - 65535].')
                    else:
                        self.module.fail_json(
                            msg='Error: The src_port_begin is not digit.')

                if self.src_port_end:
                    if self.src_port_end.isdigit():
                        if int(self.src_port_end) < 0 or int(self.src_port_end) > 65535:
                            self.module.fail_json(
                                msg='Error: The value of src_port_end is out of [0 - 65535].')
                    else:
                        self.module.fail_json(
                            msg='Error: The src_port_end is not digit.')

                if self.src_port_pool_name:
                    if len(self.src_port_pool_name) < 1 or len(self.src_port_pool_name) > 32:
                        self.module.fail_json(
                            msg='Error: The len of src_port_pool_name is out of [1 - 32].')

                if self.dest_port_op:
                    if self.dest_port_op == "lt":
                        if not self.dest_port_end:
                            self.module.fail_json(
                                msg='Error: The dest_port_end must input.')
                        if self.dest_port_begin:
                            self.module.fail_json(
                                msg='Error: The dest_port_begin should not input.')
                    if self.dest_port_op == "eq" or self.dest_port_op == "gt":
                        if not self.dest_port_begin:
                            self.module.fail_json(
                                msg='Error: The dest_port_begin must input.')
                        if self.dest_port_end:
                            self.module.fail_json(
                                msg='Error: The dest_port_end should not input.')
                    if self.dest_port_op == "range":
                        if not self.dest_port_begin or not self.dest_port_end:
                            self.module.fail_json(
                                msg='Error: The dest_port_begin and dest_port_end must input.')

                if self.dest_port_begin:
                    if self.dest_port_begin.isdigit():
                        if int(self.dest_port_begin) < 0 or int(self.dest_port_begin) > 65535:
                            self.module.fail_json(
                                msg='Error: The value of dest_port_begin is out of [0 - 65535].')
                    else:
                        self.module.fail_json(
                            msg='Error: The dest_port_begin is not digit.')

                if self.dest_port_end:
                    if self.dest_port_end.isdigit():
                        if int(self.dest_port_end) < 0 or int(self.dest_port_end) > 65535:
                            self.module.fail_json(
                                msg='Error: The value of dest_port_end is out of [0 - 65535].')
                    else:
                        self.module.fail_json(
                            msg='Error: The dest_port_end is not digit.')

                if self.dest_port_pool_name:
                    if len(self.dest_port_pool_name) < 1 or len(self.dest_port_pool_name) > 32:
                        self.module.fail_json(
                            msg='Error: The len of dest_port_pool_name is out of [1 - 32].')

                if self.precedence:
                    if self.precedence.isdigit():
                        if int(self.precedence) < 0 or int(self.precedence) > 7:
                            self.module.fail_json(
                                msg='Error: The value of precedence is out of [0 - 7].')
                    else:
                        self.module.fail_json(
                            msg='Error: The precedence is not digit.')

                if self.tos:
                    if self.tos.isdigit():
                        if int(self.tos) < 0 or int(self.tos) > 15:
                            self.module.fail_json(
                                msg='Error: The value of tos is out of [0 - 15].')
                    else:
                        self.module.fail_json(
                            msg='Error: The tos is not digit.')

                if self.dscp:
                    if self.dscp.isdigit():
                        if int(self.dscp) < 0 or int(self.dscp) > 63:
                            self.module.fail_json(
                                msg='Error: The value of dscp is out of [0 - 63].')
                    else:
                        self.module.fail_json(
                            msg='Error: The dscp is not digit.')

                if self.icmp_type:
                    if self.icmp_type.isdigit():
                        if int(self.icmp_type) < 0 or int(self.icmp_type) > 255:
                            self.module.fail_json(
                                msg='Error: The value of icmp_type is out of [0 - 255].')
                    else:
                        self.module.fail_json(
                            msg='Error: The icmp_type is not digit.')

                if self.icmp_code:
                    if self.icmp_code.isdigit():
                        if int(self.icmp_code) < 0 or int(self.icmp_code) > 255:
                            self.module.fail_json(
                                msg='Error: The value of icmp_code is out of [0 - 255].')
                    else:
                        self.module.fail_json(
                            msg='Error: The icmp_code is not digit.')

                if self.vrf_name:
                    if len(self.vrf_name) < 1 or len(self.vrf_name) > 31:
                        self.module.fail_json(
                            msg='Error: The len of vrf_name is out of [1 - 31].')

                if self.syn_flag:
                    if self.syn_flag.isdigit():
                        if int(self.syn_flag) < 0 or int(self.syn_flag) > 63:
                            self.module.fail_json(
                                msg='Error: The value of syn_flag is out of [0 - 63].')
                    else:
                        self.module.fail_json(
                            msg='Error: The syn_flag is not digit.')

                if self.tcp_flag_mask:
                    if self.tcp_flag_mask.isdigit():
                        if int(self.tcp_flag_mask) < 0 or int(self.tcp_flag_mask) > 63:
                            self.module.fail_json(
                                msg='Error: The value of tcp_flag_mask is out of [0 - 63].')
                    else:
                        self.module.fail_json(
                            msg='Error: The tcp_flag_mask is not digit.')

                if self.time_range:
                    if len(self.time_range) < 1 or len(self.time_range) > 32:
                        self.module.fail_json(
                            msg='Error: The len of time_range is out of [1 - 32].')

                if self.rule_description:
                    if len(self.rule_description) < 1 or len(self.rule_description) > 127:
                        self.module.fail_json(
                            msg='Error: The len of rule_description is out of [1 - 127].')

                if self.igmp_type:
                    self.get_igmp_type_num()

                conf_str = CE_GET_ACL_ADVANCE_RULE_HEADER % self.acl_name

                if self.rule_id:
                    conf_str += "<aclRuleID></aclRuleID>"
                if self.rule_action:
                    conf_str += "<aclAction></aclAction>"
                if self.protocol:
                    conf_str += "<aclProtocol></aclProtocol>"
                if self.source_ip:
                    conf_str += "<aclSourceIp></aclSourceIp>"
                if self.src_wild:
                    conf_str += "<aclSrcWild></aclSrcWild>"
                if self.src_pool_name:
                    conf_str += "<aclSPoolName></aclSPoolName>"
                if self.dest_ip:
                    conf_str += "<aclDestIp></aclDestIp>"
                if self.dest_wild:
                    conf_str += "<aclDestWild></aclDestWild>"
                if self.dest_pool_name:
                    conf_str += "<aclDPoolName></aclDPoolName>"
                if self.src_port_op:
                    conf_str += "<aclSrcPortOp></aclSrcPortOp>"
                if self.src_port_begin:
                    conf_str += "<aclSrcPortBegin></aclSrcPortBegin>"
                if self.src_port_end:
                    conf_str += "<aclSrcPortEnd></aclSrcPortEnd>"
                if self.src_port_pool_name:
                    conf_str += "<aclSPortPoolName></aclSPortPoolName>"
                if self.dest_port_op:
                    conf_str += "<aclDestPortOp></aclDestPortOp>"
                if self.dest_port_begin:
                    conf_str += "<aclDestPortB></aclDestPortB>"
                if self.dest_port_end:
                    conf_str += "<aclDestPortE></aclDestPortE>"
                if self.dest_port_pool_name:
                    conf_str += "<aclDPortPoolName></aclDPortPoolName>"
                if self.frag_type:
                    conf_str += "<aclFragType></aclFragType>"
                if self.precedence:
                    conf_str += "<aclPrecedence></aclPrecedence>"
                if self.tos:
                    conf_str += "<aclTos></aclTos>"
                if self.dscp:
                    conf_str += "<aclDscp></aclDscp>"
                if self.icmp_name:
                    conf_str += "<aclIcmpName></aclIcmpName>"
                if self.icmp_type:
                    conf_str += "<aclIcmpType></aclIcmpType>"
                if self.icmp_code:
                    conf_str += "<aclIcmpCode></aclIcmpCode>"
                conf_str += "<aclTtlExpired></aclTtlExpired>"
                if self.vrf_name:
                    conf_str += "<vrfName></vrfName>"
                if self.syn_flag:
                    conf_str += "<aclSynFlag></aclSynFlag>"
                if self.tcp_flag_mask:
                    conf_str += "<aclTcpFlagMask></aclTcpFlagMask>"
                conf_str += "<aclEstablished></aclEstablished>"
                if self.time_range:
                    conf_str += "<aclTimeName></aclTimeName>"
                if self.rule_description:
                    conf_str += "<aclRuleDescription></aclRuleDescription>"
                if self.igmp_type:
                    conf_str += "<aclIgmpType></aclIgmpType>"
                conf_str += "<aclLogFlag></aclLogFlag>"

                conf_str += CE_GET_ACL_ADVANCE_RULE_TAIL
                recv_xml = self.netconf_get_config(conf_str=conf_str)

                if "<data/>" in recv_xml:
                    find_flag = False

                else:
                    xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                        replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                        replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

                    root = ElementTree.fromstring(xml_str)

                    # parse advance rule
                    adv_rule_info = root.findall(
                        "acl/aclGroups/aclGroup/aclRuleAdv4s/aclRuleAdv4")
                    if adv_rule_info:
                        for tmp in adv_rule_info:
                            tmp_dict = dict()
                            for site in tmp:
                                if site.tag in ["aclRuleName", "aclRuleID", "aclAction", "aclProtocol", "aclSourceIp",
                                                "aclSrcWild", "aclSPoolName", "aclDestIp", "aclDestWild",
                                                "aclDPoolName", "aclSrcPortOp", "aclSrcPortBegin", "aclSrcPortEnd",
                                                "aclSPortPoolName", "aclDestPortOp", "aclDestPortB", "aclDestPortE",
                                                "aclDPortPoolName", "aclFragType", "aclPrecedence", "aclTos",
                                                "aclDscp", "aclIcmpName", "aclIcmpType", "aclIcmpCode", "aclTtlExpired",
                                                "vrfName", "aclSynFlag", "aclTcpFlagMask", "aclEstablished",
                                                "aclTimeName", "aclRuleDescription", "aclIgmpType", "aclLogFlag"]:
                                    tmp_dict[site.tag] = site.text

                            self.cur_advance_rule_cfg[
                                "adv_rule_info"].append(tmp_dict)

                    if self.cur_advance_rule_cfg["adv_rule_info"]:
                        for tmp in self.cur_advance_rule_cfg["adv_rule_info"]:
                            find_flag = True

                            if self.rule_name and tmp.get("aclRuleName") != self.rule_name:
                                find_flag = False
                            if self.rule_id and tmp.get("aclRuleID") != self.rule_id:
                                find_flag = False
                            if self.rule_action and tmp.get("aclAction") != self.rule_action:
                                find_flag = False
                            if self.protocol and tmp.get("aclProtocol") != self.protocol_num:
                                find_flag = False
                            if self.source_ip:
                                tmp_src_ip = self.source_ip.split(".")
                                tmp_src_wild = self.src_wild.split(".")
                                tmp_addr_item = []
                                for idx in range(4):
                                    item1 = 255 - int(tmp_src_wild[idx])
                                    item2 = item1 & int(tmp_src_ip[idx])
                                    tmp_addr_item.append(item2)
                                tmp_addr = "%s.%s.%s.%s" % (tmp_addr_item[0], tmp_addr_item[1],
                                                            tmp_addr_item[2], tmp_addr_item[3])
                                if tmp_addr != tmp.get("aclSourceIp"):
                                    find_flag = False
                            if self.src_wild and tmp.get("aclSrcWild") != self.src_wild:
                                find_flag = False
                            if self.src_pool_name and tmp.get("aclSPoolName") != self.src_pool_name:
                                find_flag = False
                            if self.dest_ip:
                                tmp_src_ip = self.dest_ip.split(".")
                                tmp_src_wild = self.dest_wild.split(".")
                                tmp_addr_item = []
                                for idx in range(4):
                                    item1 = 255 - int(tmp_src_wild[idx])
                                    item2 = item1 & int(tmp_src_ip[idx])
                                    tmp_addr_item.append(item2)
                                tmp_addr = "%s.%s.%s.%s" % (tmp_addr_item[0], tmp_addr_item[1],
                                                            tmp_addr_item[2], tmp_addr_item[3])
                                if tmp_addr != tmp.get("aclDestIp"):
                                    find_flag = False
                            if self.dest_wild and tmp.get("aclDestWild") != self.dest_wild:
                                find_flag = False
                            if self.dest_pool_name and tmp.get("aclDPoolName") != self.dest_pool_name:
                                find_flag = False
                            if self.src_port_op and tmp.get("aclSrcPortOp") != self.src_port_op:
                                find_flag = False
                            if self.src_port_begin and tmp.get("aclSrcPortBegin") != self.src_port_begin:
                                find_flag = False
                            if self.src_port_end and tmp.get("aclSrcPortEnd") != self.src_port_end:
                                find_flag = False
                            if self.src_port_pool_name and tmp.get("aclSPortPoolName") != self.src_port_pool_name:
                                find_flag = False
                            if self.dest_port_op and tmp.get("aclDestPortOp") != self.dest_port_op:
                                find_flag = False
                            if self.dest_port_begin and tmp.get("aclDestPortB") != self.dest_port_begin:
                                find_flag = False
                            if self.dest_port_end and tmp.get("aclDestPortE") != self.dest_port_end:
                                find_flag = False
                            if self.dest_port_pool_name and tmp.get("aclDPortPoolName") != self.dest_port_pool_name:
                                find_flag = False
                            frag_type = "clear_fragment" if tmp.get("aclFragType") is None else tmp.get("aclFragType")
                            if self.frag_type and frag_type != self.frag_type:
                                find_flag = False
                            if self.precedence and tmp.get("aclPrecedence") != self.precedence:
                                find_flag = False
                            if self.tos and tmp.get("aclTos") != self.tos:
                                find_flag = False
                            if self.dscp and tmp.get("aclDscp") != self.dscp:
                                find_flag = False
                            if self.icmp_name and tmp.get("aclIcmpName") != self.icmp_name:
                                find_flag = False
                            if self.icmp_type and tmp.get("aclIcmpType") != self.icmp_type:
                                find_flag = False
                            if self.icmp_code and tmp.get("aclIcmpCode") != self.icmp_code:
                                find_flag = False
                            if tmp.get("aclTtlExpired").lower() != str(self.ttl_expired).lower():
                                find_flag = False
                            if self.vrf_name and tmp.get("vrfName") != self.vrf_name:
                                find_flag = False
                            if self.syn_flag and tmp.get("aclSynFlag") != self.syn_flag:
                                find_flag = False
                            if self.tcp_flag_mask and tmp.get("aclTcpFlagMask") != self.tcp_flag_mask:
                                find_flag = False
                            if self.protocol == "tcp" and \
                                    tmp.get("aclEstablished").lower() != str(self.established).lower():
                                find_flag = False
                            if self.time_range and tmp.get("aclTimeName") != self.time_range:
                                find_flag = False
                            if self.rule_description and tmp.get("aclRuleDescription") != self.rule_description:
                                find_flag = False
                            if self.igmp_type and tmp.get("aclIgmpType") != self.igmp_type_num:
                                find_flag = False
                            if tmp.get("aclLogFlag").lower() != str(self.log_flag).lower():
                                find_flag = False

                            if find_flag:
                                break
                    else:
                        find_flag = False

                if self.state == "present":
                    need_cfg = bool(not find_flag)
                elif self.state == "absent":
                    need_cfg = bool(find_flag)
                else:
                    need_cfg = False

        self.cur_advance_rule_cfg["need_cfg"] = need_cfg

    def get_proposed(self):
        """ Get proposed state """

        self.proposed["state"] = self.state

        if self.acl_name:
            self.proposed["acl_name"] = self.acl_name
        if self.acl_num:
            self.proposed["acl_num"] = self.acl_num
        if self.acl_step:
            self.proposed["acl_step"] = self.acl_step
        if self.acl_description:
            self.proposed["acl_description"] = self.acl_description
        if self.rule_name:
            self.proposed["rule_name"] = self.rule_name
        if self.rule_id:
            self.proposed["rule_id"] = self.rule_id
        if self.rule_action:
            self.proposed["rule_action"] = self.rule_action
        if self.protocol:
            self.proposed["protocol"] = self.protocol
        if self.source_ip:
            self.proposed["source_ip"] = self.source_ip
        if self.src_mask:
            self.proposed["src_mask"] = self.src_mask
        if self.src_pool_name:
            self.proposed["src_pool_name"] = self.src_pool_name
        if self.dest_ip:
            self.proposed["dest_ip"] = self.dest_ip
        if self.dest_mask:
            self.proposed["dest_mask"] = self.dest_mask
        if self.dest_pool_name:
            self.proposed["dest_pool_name"] = self.dest_pool_name
        if self.src_port_op:
            self.proposed["src_port_op"] = self.src_port_op
        if self.src_port_begin:
            self.proposed["src_port_begin"] = self.src_port_begin
        if self.src_port_end:
            self.proposed["src_port_end"] = self.src_port_end
        if self.src_port_pool_name:
            self.proposed["src_port_pool_name"] = self.src_port_pool_name
        if self.dest_port_op:
            self.proposed["dest_port_op"] = self.dest_port_op
        if self.dest_port_begin:
            self.proposed["dest_port_begin"] = self.dest_port_begin
        if self.dest_port_end:
            self.proposed["dest_port_end"] = self.dest_port_end
        if self.dest_port_pool_name:
            self.proposed["dest_port_pool_name"] = self.dest_port_pool_name
        if self.frag_type:
            self.proposed["frag_type"] = self.frag_type
        if self.precedence:
            self.proposed["precedence"] = self.precedence
        if self.tos:
            self.proposed["tos"] = self.tos
        if self.dscp:
            self.proposed["dscp"] = self.dscp
        if self.icmp_name:
            self.proposed["icmp_name"] = self.icmp_name
        if self.icmp_type:
            self.proposed["icmp_type"] = self.icmp_type
        if self.icmp_code:
            self.proposed["icmp_code"] = self.icmp_code
        if self.ttl_expired:
            self.proposed["ttl_expired"] = self.ttl_expired
        if self.vrf_name:
            self.proposed["vrf_name"] = self.vrf_name
        if self.syn_flag:
            self.proposed["syn_flag"] = self.syn_flag
        if self.tcp_flag_mask:
            self.proposed["tcp_flag_mask"] = self.tcp_flag_mask
        self.proposed["established"] = self.established
        if self.time_range:
            self.proposed["time_range"] = self.time_range
        if self.rule_description:
            self.proposed["rule_description"] = self.rule_description
        if self.igmp_type:
            self.proposed["igmp_type"] = self.igmp_type
        self.proposed["log_flag"] = self.log_flag

    def get_existing(self):
        """ Get existing state """

        self.existing["acl_info"] = self.cur_acl_cfg["acl_info"]
        self.existing["adv_rule_info"] = self.cur_advance_rule_cfg[
            "adv_rule_info"]

    def get_end_state(self):
        """ Get end state """

        self.check_acl_args()
        self.end_state["acl_info"] = self.cur_acl_cfg["acl_info"]

        self.check_advance_rule_args()
        self.end_state["adv_rule_info"] = self.cur_advance_rule_cfg[
            "adv_rule_info"]
        if self.end_state == self.existing:
            self.changed = False
            self.updates_cmd = list()

    def merge_acl(self):
        """ Merge acl operation """

        conf_str = CE_MERGE_ACL_HEADER % self.acl_name

        if self.acl_type:
            conf_str += "<aclType>%s</aclType>" % self.acl_type
        if self.acl_num:
            conf_str += "<aclNumber>%s</aclNumber>" % self.acl_num
        if self.acl_step:
            conf_str += "<aclStep>%s</aclStep>" % self.acl_step
        if self.acl_description:
            conf_str += "<aclDescription>%s</aclDescription>" % self.acl_description

        conf_str += CE_MERGE_ACL_TAIL

        recv_xml = self.netconf_set_config(conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            self.module.fail_json(msg='Error: Merge acl failed.')

        if self.acl_name.isdigit():
            cmd = "acl number %s" % self.acl_name
        else:
            if self.acl_type and not self.acl_num:
                cmd = "acl name %s %s" % (self.acl_name, self.acl_type.lower())
            elif self.acl_type and self.acl_num:
                cmd = "acl name %s number %s" % (self.acl_name, self.acl_num)
            elif not self.acl_type and self.acl_num:
                cmd = "acl name %s number %s" % (self.acl_name, self.acl_num)
        self.updates_cmd.append(cmd)

        if self.acl_description:
            cmd = "description %s" % self.acl_description
            self.updates_cmd.append(cmd)

        if self.acl_step:
            cmd = "step %s" % self.acl_step
            self.updates_cmd.append(cmd)

        self.changed = True

    def delete_acl(self):
        """ Delete acl operation """

        conf_str = CE_DELETE_ACL_HEADER % self.acl_name

        if self.acl_type:
            conf_str += "<aclType>%s</aclType>" % self.acl_type
        if self.acl_num:
            conf_str += "<aclNumber>%s</aclNumber>" % self.acl_num
        if self.acl_step:
            conf_str += "<aclStep>%s</aclStep>" % self.acl_step
        if self.acl_description:
            conf_str += "<aclDescription>%s</aclDescription>" % self.acl_description

        conf_str += CE_DELETE_ACL_TAIL

        recv_xml = self.netconf_set_config(conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            self.module.fail_json(msg='Error: Delete acl failed.')

        if self.acl_description:
            cmd = "undo description"
            self.updates_cmd.append(cmd)

        if self.acl_step:
            cmd = "undo step"
            self.updates_cmd.append(cmd)

        if self.acl_name.isdigit():
            cmd = "undo acl number %s" % self.acl_name
        else:
            cmd = "undo acl name %s" % self.acl_name
        self.updates_cmd.append(cmd)

        self.changed = True

    def merge_adv_rule(self):
        """ Merge advance rule operation """

        conf_str = CE_MERGE_ACL_ADVANCE_RULE_HEADER % (
            self.acl_name, self.rule_name)

        if self.rule_id:
            conf_str += "<aclRuleID>%s</aclRuleID>" % self.rule_id
        if self.rule_action:
            conf_str += "<aclAction>%s</aclAction>" % self.rule_action
        if self.protocol:
            conf_str += "<aclProtocol>%s</aclProtocol>" % self.protocol_num
        if self.source_ip:
            conf_str += "<aclSourceIp>%s</aclSourceIp>" % self.source_ip
        if self.src_wild:
            conf_str += "<aclSrcWild>%s</aclSrcWild>" % self.src_wild
        if self.src_pool_name:
            conf_str += "<aclSPoolName>%s</aclSPoolName>" % self.src_pool_name
        if self.dest_ip:
            conf_str += "<aclDestIp>%s</aclDestIp>" % self.dest_ip
        if self.dest_wild:
            conf_str += "<aclDestWild>%s</aclDestWild>" % self.dest_wild
        if self.dest_pool_name:
            conf_str += "<aclDPoolName>%s</aclDPoolName>" % self.dest_pool_name
        if self.src_port_op:
            conf_str += "<aclSrcPortOp>%s</aclSrcPortOp>" % self.src_port_op
        if self.src_port_begin:
            conf_str += "<aclSrcPortBegin>%s</aclSrcPortBegin>" % self.src_port_begin
        if self.src_port_end:
            conf_str += "<aclSrcPortEnd>%s</aclSrcPortEnd>" % self.src_port_end
        if self.src_port_pool_name:
            conf_str += "<aclSPortPoolName>%s</aclSPortPoolName>" % self.src_port_pool_name
        if self.dest_port_op:
            conf_str += "<aclDestPortOp>%s</aclDestPortOp>" % self.dest_port_op
        if self.dest_port_begin:
            conf_str += "<aclDestPortB>%s</aclDestPortB>" % self.dest_port_begin
        if self.dest_port_end:
            conf_str += "<aclDestPortE>%s</aclDestPortE>" % self.dest_port_end
        if self.dest_port_pool_name:
            conf_str += "<aclDPortPoolName>%s</aclDPortPoolName>" % self.dest_port_pool_name
        if self.frag_type:
            conf_str += "<aclFragType>%s</aclFragType>" % self.frag_type
        if self.precedence:
            conf_str += "<aclPrecedence>%s</aclPrecedence>" % self.precedence
        if self.tos:
            conf_str += "<aclTos>%s</aclTos>" % self.tos
        if self.dscp:
            conf_str += "<aclDscp>%s</aclDscp>" % self.dscp
        if self.icmp_name:
            conf_str += "<aclIcmpName>%s</aclIcmpName>" % self.icmp_name
        if self.icmp_type:
            conf_str += "<aclIcmpType>%s</aclIcmpType>" % self.icmp_type
        if self.icmp_code:
            conf_str += "<aclIcmpCode>%s</aclIcmpCode>" % self.icmp_code
        conf_str += "<aclTtlExpired>%s</aclTtlExpired>" % str(self.ttl_expired).lower()
        if self.vrf_name:
            conf_str += "<vrfName>%s</vrfName>" % self.vrf_name
        if self.syn_flag:
            conf_str += "<aclSynFlag>%s</aclSynFlag>" % self.syn_flag
        if self.tcp_flag_mask:
            conf_str += "<aclTcpFlagMask>%s</aclTcpFlagMask>" % self.tcp_flag_mask
        if self.protocol == "tcp":
            conf_str += "<aclEstablished>%s</aclEstablished>" % str(self.established).lower()
        if self.time_range:
            conf_str += "<aclTimeName>%s</aclTimeName>" % self.time_range
        if self.rule_description:
            conf_str += "<aclRuleDescription>%s</aclRuleDescription>" % self.rule_description
        if self.igmp_type:
            conf_str += "<aclIgmpType>%s</aclIgmpType>" % self.igmp_type_num
        conf_str += "<aclLogFlag>%s</aclLogFlag>" % str(self.log_flag).lower()

        conf_str += CE_MERGE_ACL_ADVANCE_RULE_TAIL

        recv_xml = self.netconf_set_config(conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            self.module.fail_json(msg='Error: Merge acl base rule failed.')

        if self.rule_action and self.protocol:
            cmd = "rule"
            if self.rule_id:
                cmd += " %s" % self.rule_id
            cmd += " %s" % self.rule_action
            cmd += " %s" % self.protocol
            if self.dscp:
                cmd += " dscp %s" % self.dscp
            if self.tos:
                cmd += " tos %s" % self.tos
            if self.source_ip and self.src_wild:
                cmd += " source %s %s" % (self.source_ip, self.src_wild)
            if self.src_pool_name:
                cmd += " source-pool %s" % self.src_pool_name
            if self.src_port_op:
                cmd += " source-port"
                if self.src_port_op == "lt":
                    cmd += " lt %s" % self.src_port_end
                elif self.src_port_op == "eq":
                    cmd += " eq %s" % self.src_port_begin
                elif self.src_port_op == "gt":
                    cmd += " gt %s" % self.src_port_begin
                elif self.src_port_op == "range":
                    cmd += " range %s %s" % (self.src_port_begin,
                                             self.src_port_end)
            if self.src_port_pool_name:
                cmd += " source-port-pool %s" % self.src_port_pool_name
            if self.dest_ip and self.dest_wild:
                cmd += " destination %s %s" % (self.dest_ip, self.dest_wild)
            if self.dest_pool_name:
                cmd += " destination-pool %s" % self.dest_pool_name
            if self.dest_port_op:
                cmd += " destination-port"
                if self.dest_port_op == "lt":
                    cmd += " lt %s" % self.dest_port_end
                elif self.dest_port_op == "eq":
                    cmd += " eq %s" % self.dest_port_begin
                elif self.dest_port_op == "gt":
                    cmd += " gt %s" % self.dest_port_begin
                elif self.dest_port_op == "range":
                    cmd += " range %s %s" % (self.dest_port_begin,
                                             self.dest_port_end)
            if self.dest_port_pool_name:
                cmd += " destination-port-pool %s" % self.dest_port_pool_name
            if self.frag_type == "fragment":
                cmd += " fragment-type fragment"
            if self.precedence:
                cmd += " precedence %s" % self.precedence_name[self.precedence]

            if self.protocol == "icmp":
                if self.icmp_name:
                    cmd += " icmp-type %s" % self.icmp_name
                elif self.icmp_type and self.icmp_code:
                    cmd += " icmp-type %s %s" % (self.icmp_type, self.icmp_code)
                elif self.icmp_type:
                    cmd += " icmp-type %s" % self.icmp_type
            if self.protocol == "tcp":
                if self.syn_flag:
                    cmd += " tcp-flag %s" % self.syn_flag
                if self.tcp_flag_mask:
                    cmd += " mask %s" % self.tcp_flag_mask
                if self.established:
                    cmd += " established"
            if self.protocol == "igmp":
                if self.igmp_type:
                    cmd += " igmp-type %s" % self.igmp_type
            if self.time_range:
                cmd += " time-range %s" % self.time_range
            if self.vrf_name:
                cmd += " vpn-instance %s" % self.vrf_name
            if self.ttl_expired:
                cmd += " ttl-expired"
            if self.log_flag:
                cmd += " logging"
            self.updates_cmd.append(cmd)

        if self.rule_description:
            cmd = "rule %s description %s" % (
                self.rule_id, self.rule_description)
            self.updates_cmd.append(cmd)

        self.changed = True

    def delete_adv_rule(self):
        """ Delete advance rule operation """

        conf_str = CE_DELETE_ACL_ADVANCE_RULE_HEADER % (
            self.acl_name, self.rule_name)

        if self.rule_id:
            conf_str += "<aclRuleID>%s</aclRuleID>" % self.rule_id
        if self.rule_action:
            conf_str += "<aclAction>%s</aclAction>" % self.rule_action
        if self.protocol:
            conf_str += "<aclProtocol>%s</aclProtocol>" % self.protocol_num
        if self.source_ip:
            conf_str += "<aclSourceIp>%s</aclSourceIp>" % self.source_ip
        if self.src_wild:
            conf_str += "<aclSrcWild>%s</aclSrcWild>" % self.src_wild
        if self.src_pool_name:
            conf_str += "<aclSPoolName>%s</aclSPoolName>" % self.src_pool_name
        if self.dest_ip:
            conf_str += "<aclDestIp>%s</aclDestIp>" % self.dest_ip
        if self.dest_wild:
            conf_str += "<aclDestWild>%s</aclDestWild>" % self.dest_wild
        if self.dest_pool_name:
            conf_str += "<aclDPoolName>%s</aclDPoolName>" % self.dest_pool_name
        if self.src_port_op:
            conf_str += "<aclSrcPortOp>%s</aclSrcPortOp>" % self.src_port_op
        if self.src_port_begin:
            conf_str += "<aclSrcPortBegin>%s</aclSrcPortBegin>" % self.src_port_begin
        if self.src_port_end:
            conf_str += "<aclSrcPortEnd>%s</aclSrcPortEnd>" % self.src_port_end
        if self.src_port_pool_name:
            conf_str += "<aclSPortPoolName>%s</aclSPortPoolName>" % self.src_port_pool_name
        if self.dest_port_op:
            conf_str += "<aclDestPortOp>%s</aclDestPortOp>" % self.dest_port_op
        if self.dest_port_begin:
            conf_str += "<aclDestPortB>%s</aclDestPortB>" % self.dest_port_begin
        if self.dest_port_end:
            conf_str += "<aclDestPortE>%s</aclDestPortE>" % self.dest_port_end
        if self.dest_port_pool_name:
            conf_str += "<aclDPortPoolName>%s</aclDPortPoolName>" % self.dest_port_pool_name
        if self.frag_type:
            conf_str += "<aclFragType>%s</aclFragType>" % self.frag_type
        if self.precedence:
            conf_str += "<aclPrecedence>%s</aclPrecedence>" % self.precedence
        if self.tos:
            conf_str += "<aclTos>%s</aclTos>" % self.tos
        if self.dscp:
            conf_str += "<aclDscp>%s</aclDscp>" % self.dscp
        if self.icmp_name:
            conf_str += "<aclIcmpName>%s</aclIcmpName>" % self.icmp_name
        if self.icmp_type:
            conf_str += "<aclIcmpType>%s</aclIcmpType>" % self.icmp_type
        if self.icmp_code:
            conf_str += "<aclIcmpCode>%s</aclIcmpCode>" % self.icmp_code
        conf_str += "<aclTtlExpired>%s</aclTtlExpired>" % str(self.ttl_expired).lower()
        if self.vrf_name:
            conf_str += "<vrfName>%s</vrfName>" % self.vrf_name
        if self.syn_flag:
            conf_str += "<aclSynFlag>%s</aclSynFlag>" % self.syn_flag
        if self.tcp_flag_mask:
            conf_str += "<aclTcpFlagMask>%s</aclTcpFlagMask>" % self.tcp_flag_mask
        if self.protocol == "tcp":
            conf_str += "<aclEstablished>%s</aclEstablished>" % str(self.established).lower()
        if self.time_range:
            conf_str += "<aclTimeName>%s</aclTimeName>" % self.time_range
        if self.rule_description:
            conf_str += "<aclRuleDescription>%s</aclRuleDescription>" % self.rule_description
        if self.igmp_type:
            conf_str += "<aclIgmpType>%s</aclIgmpType>" % self.igmp_type
        conf_str += "<aclLogFlag>%s</aclLogFlag>" % str(self.log_flag).lower()

        conf_str += CE_DELETE_ACL_ADVANCE_RULE_TAIL

        recv_xml = self.netconf_set_config(conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            self.module.fail_json(msg='Error: Delete acl base rule failed.')

        if self.rule_description:
            if self.acl_name.isdigit():
                cmd = "acl number %s" % self.acl_name
            else:
                cmd = "acl name %s" % self.acl_name
            self.updates_cmd.append(cmd)

            cmd = "undo rule %s description" % self.rule_id
            self.updates_cmd.append(cmd)

        if self.rule_id:
            if self.acl_name.isdigit():
                cmd = "acl number %s" % self.acl_name
            else:
                cmd = "acl name %s" % self.acl_name
            self.updates_cmd.append(cmd)

            cmd = "undo rule %s" % self.rule_id
            self.updates_cmd.append(cmd)
        elif self.rule_action and self.protocol:
            if self.acl_name.isdigit():
                cmd = "acl number %s" % self.acl_name
            else:
                cmd = "acl name %s" % self.acl_name
            self.updates_cmd.append(cmd)

            cmd = "undo rule"
            cmd += " %s" % self.rule_action
            cmd += " %s" % self.protocol
            if self.dscp:
                cmd += " dscp %s" % self.dscp
            if self.tos:
                cmd += " tos %s" % self.tos
            if self.source_ip and self.src_mask:
                cmd += " source %s %s" % (self.source_ip, self.src_mask)
            if self.src_pool_name:
                cmd += " source-pool %s" % self.src_pool_name
            if self.src_port_op:
                cmd += " source-port"
                if self.src_port_op == "lt":
                    cmd += " lt %s" % self.src_port_end
                elif self.src_port_op == "eq":
                    cmd += " eq %s" % self.src_port_begin
                elif self.src_port_op == "gt":
                    cmd += " gt %s" % self.src_port_begin
                elif self.src_port_op == "range":
                    cmd += " range %s %s" % (self.src_port_begin,
                                             self.src_port_end)
            if self.src_port_pool_name:
                cmd += " source-port-pool %s" % self.src_port_pool_name
            if self.dest_ip and self.dest_mask:
                cmd += " destination %s %s" % (self.dest_ip, self.dest_mask)
            if self.dest_pool_name:
                cmd += " destination-pool %s" % self.dest_pool_name
            if self.dest_port_op:
                cmd += " destination-port"
                if self.dest_port_op == "lt":
                    cmd += " lt %s" % self.dest_port_end
                elif self.dest_port_op == "eq":
                    cmd += " eq %s" % self.dest_port_begin
                elif self.dest_port_op == "gt":
                    cmd += " gt %s" % self.dest_port_begin
                elif self.dest_port_op == "range":
                    cmd += " range %s %s" % (self.dest_port_begin,
                                             self.dest_port_end)
            if self.dest_port_pool_name:
                cmd += " destination-port-pool %s" % self.dest_port_pool_name
            if self.frag_type == "fragment":
                cmd += " fragment-type fragment"
            if self.precedence:
                cmd += " precedence %s" % self.precedence_name[self.precedence]
            if self.time_range:
                cmd += " time-range %s" % self.time_range
            if self.vrf_name:
                cmd += " vpn-instance %s" % self.vrf_name
            if self.ttl_expired:
                cmd += " ttl-expired"
            if self.log_flag:
                cmd += " logging"
            self.updates_cmd.append(cmd)

        self.changed = True

    def work(self):
        """ Main work function """

        self.check_acl_args()
        self.check_advance_rule_args()
        self.get_proposed()
        self.get_existing()

        if self.state == "present":
            if self.cur_acl_cfg["need_cfg"]:
                self.merge_acl()
            if self.cur_advance_rule_cfg["need_cfg"]:
                self.merge_adv_rule()

        elif self.state == "absent":
            if self.cur_advance_rule_cfg["need_cfg"]:
                self.delete_adv_rule()

        elif self.state == "delete_acl":
            if self.cur_acl_cfg["need_cfg"]:
                self.delete_acl()

        self.get_end_state()

        self.results['changed'] = self.changed
        self.results['proposed'] = self.proposed
        self.results['existing'] = self.existing
        self.results['end_state'] = self.end_state
        self.results['updates'] = self.updates_cmd

        self.module.exit_json(**self.results)


def main():
    """ Module main """

    argument_spec = dict(
        state=dict(choices=['present', 'absent',
                            'delete_acl'], default='present'),
        acl_name=dict(type='str', required=True),
        acl_num=dict(type='str'),
        acl_step=dict(type='str'),
        acl_description=dict(type='str'),
        rule_name=dict(type='str'),
        rule_id=dict(type='str'),
        rule_action=dict(choices=['permit', 'deny']),
        protocol=dict(choices=['ip', 'icmp', 'igmp',
                               'ipinip', 'tcp', 'udp', 'gre', 'ospf']),
        source_ip=dict(type='str'),
        src_mask=dict(type='str'),
        src_pool_name=dict(type='str'),
        dest_ip=dict(type='str'),
        dest_mask=dict(type='str'),
        dest_pool_name=dict(type='str'),
        src_port_op=dict(choices=['lt', 'eq', 'gt', 'range']),
        src_port_begin=dict(type='str'),
        src_port_end=dict(type='str'),
        src_port_pool_name=dict(type='str'),
        dest_port_op=dict(choices=['lt', 'eq', 'gt', 'range']),
        dest_port_begin=dict(type='str'),
        dest_port_end=dict(type='str'),
        dest_port_pool_name=dict(type='str'),
        frag_type=dict(choices=['fragment', 'clear_fragment']),
        precedence=dict(type='str'),
        tos=dict(type='str'),
        dscp=dict(type='str'),
        icmp_name=dict(choices=['unconfiged', 'echo', 'echo-reply', 'fragmentneed-DFset', 'host-redirect',
                                'host-tos-redirect', 'host-unreachable', 'information-reply', 'information-request',
                                'net-redirect', 'net-tos-redirect', 'net-unreachable', 'parameter-problem',
                                'port-unreachable', 'protocol-unreachable', 'reassembly-timeout', 'source-quench',
                                'source-route-failed', 'timestamp-reply', 'timestamp-request', 'ttl-exceeded',
                                'address-mask-reply', 'address-mask-request', 'custom']),
        icmp_type=dict(type='str'),
        icmp_code=dict(type='str'),
        ttl_expired=dict(required=False, default=False, type='bool'),
        vrf_name=dict(type='str'),
        syn_flag=dict(type='str'),
        tcp_flag_mask=dict(type='str'),
        established=dict(required=False, default=False, type='bool'),
        time_range=dict(type='str'),
        rule_description=dict(type='str'),
        igmp_type=dict(choices=['host-query', 'mrouter-adver', 'mrouter-solic', 'mrouter-termi', 'mtrace-resp',
                                'mtrace-route', 'v1host-report', 'v2host-report', 'v2leave-group', 'v3host-report']),
        log_flag=dict(required=False, default=False, type='bool')
    )

    argument_spec.update(ce_argument_spec)
    module = AdvanceAcl(argument_spec=argument_spec)
    module.work()


if __name__ == '__main__':
    main()
